@@ -1,6 +1,6 @@
 """Arbitrary precision arithmetic."""
 
-__version__ = '3.0.0'
+__version__ = '4.0.0'
 
 try:
     from ._rithm import (Fraction,
@@ -41,16 +41,8 @@ except ImportError:
         def __bool__(self) -> bool:
             return bool(self._value)
 
-        def __ceil__(self) -> int:
-            return self._value
-
-        def __floor__(self) -> int:
-            return self._value
-
-        def __floordiv__(self, other: 'Int') -> 'Int':
-            return (Int(self._value // other._value)
-                    if isinstance(other, Int)
-                    else NotImplemented)
+        def __ceil__(self) -> 'Int':
+            return self
 
         def __divmod__(self, other: 'Int') -> _Tuple['Int', 'Int']:
             return (tuple(Int(value)
@@ -63,6 +55,17 @@ except ImportError:
                     if isinstance(other, Int)
                     else NotImplemented)
 
+        def __float__(self) -> float:
+            return float(self._value)
+
+        def __floor__(self) -> 'Int':
+            return self
+
+        def __floordiv__(self, other: 'Int') -> 'Int':
+            return (Int(self._value // other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         def __ge__(self, other: 'Int') -> bool:
             return (self._value >= other._value
                     if isinstance(other, Int)
@@ -82,11 +85,19 @@ except ImportError:
         def __invert__(self) -> 'Int':
             return Int(~self._value)
 
+        def __int__(self) -> int:
+            return self._value
+
         def __le__(self, other: 'Int') -> bool:
             return (self._value <= other._value
                     if isinstance(other, Int)
                     else NotImplemented)
 
+        def __lshift__(self, other: 'Int') -> 'Int':
+            return (Int(self._value << other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         def __lt__(self, other: 'Int') -> bool:
             return (self._value < other._value
                     if isinstance(other, Int)
@@ -123,6 +134,11 @@ except ImportError:
         def __repr__(self) -> str:
             return f'rithm.Int(\'{self._value}\')'
 
+        def __rshift__(self, other: 'Int') -> 'Int':
+            return (Int(self._value >> other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         def __setstate__(self, state: int) -> None:
             self._value = state
 
@@ -139,8 +155,8 @@ except ImportError:
                     if isinstance(other, Int)
                     else NotImplemented)
 
-        def __trunc__(self) -> int:
-            return self._value
+        def __trunc__(self) -> 'Int':
+            return self
 
 
     _ONE = Int('1')
@@ -159,17 +175,28 @@ except ImportError:
         __slots__ = '_denominator', '_numerator'
 
         def __new__(cls,
-                    numerator: Int = _ZERO,
-                    denominator: Int = _ONE,
+                    _numerator: _Union[Int, float] = _ZERO,
+                    _denominator: _Optional[Int] = None,
                     *,
                     _normalize: bool = True) -> 'Fraction':
             self = super().__new__(cls)
-            if not isinstance(numerator, Int):
-                raise TypeError(f'First argument should be of type {Int}, '
-                                f'but found: {type(denominator)}.')
-            if not isinstance(denominator, Int):
+            if _denominator is None:
+                if isinstance(_numerator, float):
+                    raw_numerator, raw_denominator = (
+                        _numerator.as_integer_ratio())
+                    numerator, denominator = (Int(raw_numerator),
+                                              Int(raw_denominator))
+                elif isinstance(_numerator, Int):
+                    numerator, denominator = _numerator, _ONE
+                else:
+                    raise TypeError('First argument should be of '
+                                    f'type {Int} or {float}, '
+                                    f'but found: {type(_numerator)}.')
+            elif isinstance(_denominator, Int):
+                numerator, denominator = _numerator, _denominator
+            else:
                 raise TypeError(f'Denominator should be of type {Int}, '
-                                f'but found: {type(denominator)}.')
+                                f'but found: {type(_denominator)}.')
             if not denominator:
                 raise ZeroDivisionError('Denominator should not be zero.')
             if _normalize:
@@ -210,6 +237,9 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
+        def __float__(self) -> float:
+            return self._numerator._value / self._denominator._value
+
         def __ge__(self, other: 'Fraction') -> bool:
             return (self.numerator * other.denominator
                     >= other.numerator * self.denominator
