@@ -1,5 +1,5 @@
 use std::cmp::Ordering;
-use std::convert::TryFrom;
+use std::convert::{FloatToInt, TryFrom};
 use std::fmt::{Debug, Display, Formatter};
 use std::iter::Peekable;
 use std::mem::size_of;
@@ -8,14 +8,15 @@ use std::str::Chars;
 
 use crate::digits::*;
 use crate::traits::{
-    Abs, CheckedDiv, CheckedDivEuclid, CheckedDivRem, CheckedDivRemEuclid, CheckedPow,
-    CheckedPowRemEuclid, CheckedRem, CheckedRemEuclid, CheckedRemEuclidInv, DivEuclid, DivRem,
-    DivRemEuclid, DoublePrecisionOf, FromStrRadix, Gcd, Oppose, OppositionOf, Oppositive, Pow,
-    RemEuclid, Unitary, Zeroable,
+    Abs, AssigningShiftingLeftMonoid, BitwiseNegatableUnaryAlgebra, CheckedDiv, CheckedDivAsF32,
+    CheckedDivAsF64, CheckedDivEuclid, CheckedDivRem, CheckedDivRemEuclid, CheckedPow,
+    CheckedPowRemEuclid, CheckedRem, CheckedRemEuclid, CheckedRemEuclidInv, CheckedShl, CheckedShr,
+    DivEuclid, DivRem, DivRemEuclid, DoublePrecisionOf, Float, FromStrRadix, Gcd, Oppose,
+    OppositionOf, Oppositive, Pow, RemEuclid, Unitary, Zeroable,
 };
 use crate::utils;
 
-#[derive(Clone, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub struct BigInt<Digit, const SEPARATOR: char, const SHIFT: usize> {
     sign: Sign,
     digits: Vec<Digit>,
@@ -23,54 +24,116 @@ pub struct BigInt<Digit, const SEPARATOR: char, const SHIFT: usize> {
 
 const MAX_REPRESENTABLE_BASE: u8 = 36;
 
-pub struct BigIntParsingError {
-    kind: BigIntParsingErrorKind,
+pub enum FromFloatConversionError {
+    Infinity,
+    NaN,
 }
 
-impl Debug for BigIntParsingError {
+impl FromFloatConversionError {
+    fn description(&self) -> &str {
+        match self {
+            FromFloatConversionError::Infinity => "Conversion of infinity is undefined.",
+            FromFloatConversionError::NaN => "Conversion of NaN is undefined.",
+        }
+    }
+}
+
+impl Debug for FromFloatConversionError {
     fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
-        formatter.write_str(self.kind.description().as_str())
+        formatter.write_str(self.description())
     }
 }
 
-impl Display for BigIntParsingError {
+impl Display for FromFloatConversionError {
     fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
-        Display::fmt(&self.kind.description(), formatter)
+        Display::fmt(&self.description(), formatter)
     }
 }
 
-impl BigIntParsingError {
-    pub fn kind(&self) -> &BigIntParsingErrorKind {
-        &self.kind
-    }
-}
-
-pub enum BigIntParsingErrorKind {
+pub enum ParsingError {
     StartsWithSeparator,
     ConsecutiveSeparators,
     InvalidDigit(char, u8),
     EndsWithSeparator,
 }
 
-impl BigIntParsingErrorKind {
+impl ParsingError {
     fn description(&self) -> String {
         match self {
-            BigIntParsingErrorKind::StartsWithSeparator => {
-                String::from("Should not start with separator.")
-            }
-            BigIntParsingErrorKind::InvalidDigit(character, base) => {
+            ParsingError::StartsWithSeparator => String::from("Should not start with separator."),
+            ParsingError::InvalidDigit(character, base) => {
                 format!("Invalid digit in base {}: {}.", base, character)
             }
-            BigIntParsingErrorKind::ConsecutiveSeparators => {
-                String::from("Consecutive separators found.")
-            }
-            BigIntParsingErrorKind::EndsWithSeparator => {
-                String::from("Should not end with separator.")
-            }
+            ParsingError::ConsecutiveSeparators => String::from("Consecutive separators found."),
+            ParsingError::EndsWithSeparator => String::from("Should not end with separator."),
         }
     }
 }
 
+impl Debug for ParsingError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(&self.description())
+    }
+}
+
+impl Display for ParsingError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
+pub enum ShiftError {
+    NegativeShift,
+    OutOfMemory,
+    TooLarge,
+}
+
+impl ShiftError {
+    fn description(&self) -> String {
+        match self {
+            ShiftError::NegativeShift => String::from("Shift by negative step is undefined."),
+            ShiftError::OutOfMemory => String::from("Not enough memory for shift result."),
+            ShiftError::TooLarge => String::from("Too large shift step."),
+        }
+    }
+}
+
+impl Debug for ShiftError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(&self.description())
+    }
+}
+
+impl Display for ShiftError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
+pub enum ToFloatConversionError {
+    TooLarge,
+}
+
+impl ToFloatConversionError {
+    fn description(&self) -> &str {
+        match self {
+            ToFloatConversionError::TooLarge => "Too large to convert to floating point.",
+        }
+    }
+}
+
+impl Debug for ToFloatConversionError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(self.description())
+    }
+}
+
+impl Display for ToFloatConversionError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
 impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
     BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -89,7 +152,8 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
         37,
     ];
 
-    fn new(string: &str, mut base: u8) -> Result<Self, BigIntParsingError> {
+    fn new(string: &str, mut base: u8) -> Result<Self, ParsingError> {
+        debug_assert!(is_valid_shift::<Digit, SHIFT>());
         debug_assert!(Self::ASCII_CODES_DIGIT_VALUES[SEPARATOR as usize] >= MAX_REPRESENTABLE_BASE);
         let mut characters = string.trim().chars().peekable();
         let sign = Self::parse_sign(&mut characters);
@@ -121,14 +185,9 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
     }
 
     #[inline]
-    fn parse_digits(
-        mut characters: Peekable<Chars>,
-        base: u8,
-    ) -> Result<Vec<u8>, BigIntParsingError> {
+    fn parse_digits(mut characters: Peekable<Chars>, base: u8) -> Result<Vec<u8>, ParsingError> {
         if characters.peek() == Some(&SEPARATOR) {
-            return Err(BigIntParsingError {
-                kind: BigIntParsingErrorKind::StartsWithSeparator,
-            });
+            return Err(ParsingError::StartsWithSeparator);
         }
         let mut result: Vec<u8> = Vec::new();
         let mut prev: char = SEPARATOR;
@@ -136,22 +195,16 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
             if character != SEPARATOR {
                 let digit = Self::ASCII_CODES_DIGIT_VALUES[character as usize];
                 if digit >= base {
-                    return Err(BigIntParsingError {
-                        kind: BigIntParsingErrorKind::InvalidDigit(character, base),
-                    });
+                    return Err(ParsingError::InvalidDigit(character, base));
                 }
                 result.push(digit);
             } else if prev == SEPARATOR {
-                return Err(BigIntParsingError {
-                    kind: BigIntParsingErrorKind::ConsecutiveSeparators,
-                });
+                return Err(ParsingError::ConsecutiveSeparators);
             }
             prev = character;
         }
         if prev == SEPARATOR {
-            return Err(BigIntParsingError {
-                kind: BigIntParsingErrorKind::EndsWithSeparator,
-            });
+            return Err(ParsingError::EndsWithSeparator);
         }
         result.reverse();
         Ok(result)
@@ -261,7 +314,7 @@ impl<Digit: DisplayDigit, const SEPARATOR: char, const SHIFT: usize>
 const MIDDLE_BYTE: u8 = 1u8 << (u8::BITS - 1);
 
 impl<
-        Digit: BinaryDigitConvertibleToBinary<Digit> + From<u8>,
+        Digit: BinaryDigitConvertibleToBinary<Digit> + From<u8> + Oppose,
         const SEPARATOR: char,
         const SHIFT: usize,
     > BigInt<Digit, SEPARATOR, SHIFT>
@@ -289,11 +342,12 @@ where
     }
 }
 
-impl<Digit, const SEPARATOR: char, const SHIFT: usize> BigInt<Digit, SEPARATOR, SHIFT>
+impl<Digit: Oppose, const SEPARATOR: char, const SHIFT: usize> BigInt<Digit, SEPARATOR, SHIFT>
 where
     u8: BinaryDigitConvertibleToBinary<Digit>,
 {
     pub(crate) fn from_bytes(mut bytes: Vec<u8>) -> Self {
+        debug_assert!(is_valid_shift::<Digit, SHIFT>());
         let most_significant_byte = bytes[bytes.len() - 1];
         let sign = if most_significant_byte >= MIDDLE_BYTE {
             negate_digits(&mut bytes);
@@ -651,7 +705,7 @@ impl<Digit: DisplayDigit, const SEPARATOR: char, const SHIFT: usize> Display
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn fmt(&self, formatter: &mut Formatter) -> std::fmt::Result {
-        formatter.write_str(self.to_base_string(10).as_str())
+        formatter.write_str(&self.to_base_string(10))
     }
 }
 
@@ -672,6 +726,34 @@ impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Div
     }
 }
 
+impl<
+        Digit: BinaryDigitConvertibleToFloat<f32> + BitwiseNegatableUnaryAlgebra + DivisibleDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > CheckedDivAsF32 for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Result<f32, CheckedDivApproximationError>;
+
+    fn checked_div_as_f32(self, divisor: Self) -> Self::Output {
+        checked_div_approximation::<Digit, f32, SHIFT>(&self.digits, &divisor.digits)
+            .map(|modulus| ((self.sign * divisor.sign) as f32) * modulus)
+    }
+}
+
+impl<
+        Digit: BinaryDigitConvertibleToFloat<f64> + BitwiseNegatableUnaryAlgebra + DivisibleDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > CheckedDivAsF64 for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Result<f64, CheckedDivApproximationError>;
+
+    fn checked_div_as_f64(self, divisor: Self) -> Self::Output {
+        checked_div_approximation::<Digit, f64, SHIFT>(&self.digits, &divisor.digits)
+            .map(|modulus| ((self.sign * divisor.sign) as f64) * modulus)
+    }
+}
+
 impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> DivAssign
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -723,13 +805,31 @@ impl<Digit: EuclidDivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Div
     }
 }
 
+impl<Digit: DigitConvertibleFromFloat<f64>, const SEPARATOR: char, const SHIFT: usize>
+    FloatToInt<BigInt<Digit, SEPARATOR, SHIFT>> for f32
+{
+    unsafe fn to_int_unchecked(self) -> BigInt<Digit, SEPARATOR, SHIFT> {
+        BigInt::<Digit, SEPARATOR, SHIFT>::try_from(self).unwrap_unchecked()
+    }
+}
+
+impl<Digit: DigitConvertibleFromFloat<f64>, const SEPARATOR: char, const SHIFT: usize>
+    FloatToInt<BigInt<Digit, SEPARATOR, SHIFT>> for f64
+{
+    unsafe fn to_int_unchecked(self) -> BigInt<Digit, SEPARATOR, SHIFT> {
+        BigInt::<Digit, SEPARATOR, SHIFT>::try_from(self).unwrap_unchecked()
+    }
+}
+
 impl<SourceDigit, TargetDigit, const SEPARATOR: char, const SHIFT: usize> From<SourceDigit>
     for BigInt<TargetDigit, SEPARATOR, SHIFT>
 where
-    SourceDigit: BinaryDigit + Oppose,
+    SourceDigit: BinaryDigit + Oppose + TryFrom<OppositionOf<SourceDigit>>,
     TargetDigit: BinaryDigit + Oppose + TryFrom<SourceDigit>,
+    OppositionOf<SourceDigit>: TryFrom<SourceDigit>,
 {
-    fn from(mut value: SourceDigit) -> Self {
+    fn from(value: SourceDigit) -> Self {
+        debug_assert!(is_valid_shift::<TargetDigit, SHIFT>());
         if value.is_zero() {
             Self::zero()
         } else if size_of::<SourceDigit>() < size_of::<TargetDigit>()
@@ -737,8 +837,25 @@ where
                 && utils::are_same::<SourceDigit, OppositionOf<SourceDigit>>()
                 && !utils::are_same::<TargetDigit, OppositionOf<TargetDigit>>())
         {
-            let mut value = unsafe { TargetDigit::try_from(value).unwrap_unchecked() };
-            let sign = Sign::one();
+            let mut sign = Sign::one();
+            let mut value = if utils::are_same::<SourceDigit, OppositionOf<SourceDigit>>() {
+                let value =
+                    unsafe { OppositionOf::<SourceDigit>::try_from(value).unwrap_unchecked() };
+                if value.is_negative() {
+                    sign = -sign;
+                    unsafe {
+                        TargetDigit::try_from(SourceDigit::try_from(-value).unwrap_unchecked())
+                            .unwrap_unchecked()
+                    }
+                } else {
+                    unsafe {
+                        TargetDigit::try_from(SourceDigit::try_from(value).unwrap_unchecked())
+                            .unwrap_unchecked()
+                    }
+                }
+            } else {
+                unsafe { TargetDigit::try_from(value).unwrap_unchecked() }
+            };
             let mut digits = Vec::<TargetDigit>::new();
             let digit_mask = to_digit_mask::<TargetDigit>(SHIFT);
             while !value.is_zero() {
@@ -747,7 +864,19 @@ where
             }
             Self { sign, digits }
         } else {
-            let sign = Sign::one();
+            let mut sign = Sign::one();
+            let mut value = if utils::are_same::<SourceDigit, OppositionOf<SourceDigit>>() {
+                let value =
+                    unsafe { OppositionOf::<SourceDigit>::try_from(value).unwrap_unchecked() };
+                if value.is_negative() {
+                    sign = -sign;
+                    unsafe { SourceDigit::try_from(-value).unwrap_unchecked() }
+                } else {
+                    unsafe { SourceDigit::try_from(value).unwrap_unchecked() }
+                }
+            } else {
+                value
+            };
             let mut digits = Vec::<TargetDigit>::new();
             let digit_mask = to_digit_mask::<SourceDigit>(SHIFT);
             while !value.is_zero() {
@@ -763,7 +892,7 @@ where
 impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize> FromStrRadix
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
-    type Error = BigIntParsingError;
+    type Error = ParsingError;
 
     fn from_str_radix(string: &str, radix: u32) -> Result<Self, Self::Error> {
         if (radix != 0 && radix < 2) || radix > (MAX_REPRESENTABLE_BASE as u32) {
@@ -971,10 +1100,8 @@ impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> Not
     }
 }
 
-impl<Digit, const SEPARATOR: char, const SHIFT: usize> Oppositive
+impl<Digit: OppositiveDigit, const SEPARATOR: char, const SHIFT: usize> Oppositive
     for BigInt<Digit, SEPARATOR, SHIFT>
-where
-    Digit: Zeroable,
 {
     fn is_negative(&self) -> bool {
         self.sign.is_negative()
@@ -985,7 +1112,7 @@ where
     }
 }
 
-impl<Digit: Clone + Eq + PartialOrd + Zeroable, const SEPARATOR: char, const SHIFT: usize> Ord
+impl<Digit: Clone + Eq + PartialOrd + ZeroableDigit, const SEPARATOR: char, const SHIFT: usize> Ord
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn cmp(&self, other: &Self) -> Ordering {
@@ -999,8 +1126,8 @@ impl<Digit: Clone + Eq + PartialOrd + Zeroable, const SEPARATOR: char, const SHI
     }
 }
 
-impl<Digit: Clone + PartialOrd + Zeroable, const SEPARATOR: char, const SHIFT: usize> PartialOrd
-    for BigInt<Digit, SEPARATOR, SHIFT>
+impl<Digit: Clone + PartialOrd + ZeroableDigit, const SEPARATOR: char, const SHIFT: usize>
+    PartialOrd for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn ge(&self, other: &Self) -> bool {
         self.sign > other.sign
@@ -1098,6 +1225,247 @@ impl<Digit: EuclidDivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Rem
     }
 }
 
+impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Result<Self, ShiftError>;
+
+    fn checked_shl(self, shift: Self) -> Self::Output {
+        if shift.is_negative() {
+            Err(ShiftError::NegativeShift)
+        } else if self.is_zero() {
+            Ok(self)
+        } else {
+            let (shift_quotient_digits, shift_remainder) =
+                div_rem_digits_by_digit::<Digit, SHIFT>(&shift.digits, unsafe {
+                    Digit::try_from(SHIFT).unwrap_unchecked()
+                });
+            let shift_quotient =
+                checked_reduce_digits::<Digit, usize, SHIFT>(&shift_quotient_digits)
+                    .ok_or(ShiftError::TooLarge)?;
+            if shift_quotient >= usize::MAX / size_of::<Digit>() {
+                Err(ShiftError::TooLarge)
+            } else {
+                let digits = left_shift_digits::<Digit, SHIFT>(
+                    &self.digits,
+                    shift_quotient,
+                    shift_remainder,
+                )
+                .ok_or(ShiftError::OutOfMemory)?;
+                Ok(Self {
+                    sign: self.sign,
+                    digits,
+                })
+            }
+        }
+    }
+}
+
+macro_rules! plain_signed_checked_shl_impl {
+    ($($t:ty)*) => ($(
+        impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl<$t>
+            for BigInt<Digit, SEPARATOR, SHIFT>
+        {
+            type Output = Result<Self, ShiftError>;
+
+            fn checked_shl(self, shift: $t) -> Self::Output {
+                debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
+                if shift.is_negative() {
+                    Err(ShiftError::NegativeShift)
+                } else if self.is_zero() {
+                    Ok(self)
+                } else {
+                    let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
+                    if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
+                        && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t) {
+                        Err(ShiftError::TooLarge)
+                    } else {
+                        let digits = left_shift_digits::<Digit, SHIFT>(
+                            &self.digits,
+                            shift_quotient as usize,
+                            unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
+                        )
+                        .ok_or(ShiftError::OutOfMemory)?;
+                        Ok(Self {
+                            sign: self.sign,
+                            digits,
+                        })
+                    }
+                }
+            }
+        }
+    )*)
+}
+
+plain_signed_checked_shl_impl!(i8 i16 i32 i64 i128 isize);
+
+macro_rules! plain_unsigned_checked_shl_impl {
+    ($($t:ty)*) => ($(
+        impl<Digit: LeftShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShl<$t>
+            for BigInt<Digit, SEPARATOR, SHIFT>
+        {
+            type Output = Result<Self, ShiftError>;
+
+            fn checked_shl(self, shift: $t) -> Self::Output {
+                debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
+                if self.is_zero() {
+                    Ok(self)
+                } else {
+                    let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
+                    if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
+                        && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t) {
+                        Err(ShiftError::TooLarge)
+                    } else {
+                        let digits = left_shift_digits::<Digit, SHIFT>(
+                            &self.digits,
+                            shift_quotient as usize,
+                            unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
+                        )
+                        .ok_or(ShiftError::OutOfMemory)?;
+                        Ok(Self {
+                            sign: self.sign,
+                            digits,
+                        })
+                    }
+                }
+            }
+        }
+    )*)
+}
+
+plain_unsigned_checked_shl_impl!(u8 u16 u32 u64 u128 usize);
+
+impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Result<Self, ShiftError>;
+
+    fn checked_shr(self, shift: Self) -> Self::Output {
+        if shift.is_negative() {
+            Err(ShiftError::NegativeShift)
+        } else if self.is_zero() {
+            Ok(self)
+        } else {
+            let (shift_quotient_digits, shift_remainder) =
+                div_rem_digits_by_digit::<Digit, SHIFT>(&shift.digits, unsafe {
+                    Digit::try_from(SHIFT).unwrap_unchecked()
+                });
+            let shift_quotient =
+                checked_reduce_digits::<Digit, usize, SHIFT>(&shift_quotient_digits)
+                    .ok_or(ShiftError::TooLarge)?;
+            if shift_quotient >= usize::MAX / size_of::<Digit>() {
+                Err(ShiftError::TooLarge)
+            } else if self.is_negative() {
+                let inverted = !self;
+                let digits = right_shift_digits::<Digit, SHIFT>(
+                    &inverted.digits,
+                    shift_quotient,
+                    shift_remainder,
+                );
+                Ok(!Self {
+                    sign: inverted.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                    digits,
+                })
+            } else {
+                let digits = right_shift_digits::<Digit, SHIFT>(
+                    &self.digits,
+                    shift_quotient,
+                    shift_remainder,
+                );
+                Ok(Self {
+                    sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                    digits,
+                })
+            }
+        }
+    }
+}
+
+macro_rules! plain_signed_checked_shr_impl {
+    ($($t:ty)*) => ($(
+        impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr<$t>
+            for BigInt<Digit, SEPARATOR, SHIFT>
+        {
+            type Output = Result<Self, ShiftError>;
+
+            fn checked_shr(self, shift: $t) -> Self::Output {
+                debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
+                if shift.is_negative() {
+                    Err(ShiftError::NegativeShift)
+                } else if self.is_zero() {
+                    Ok(self)
+                } else {
+                    let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
+                    if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
+                        && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t)
+                    {
+                        Err(ShiftError::TooLarge)
+                    } else if self.is_negative() {
+                        let inverted = !self;
+                        let digits = right_shift_digits::<Digit, SHIFT>(
+                            &inverted.digits,
+                            shift_quotient as usize,
+                            unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
+                        );
+                        Ok(!Self {
+                            sign: inverted.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            digits,
+                        })
+                    } else {
+                        let digits = right_shift_digits::<Digit, SHIFT>(
+                            &self.digits,
+                            shift_quotient as usize,
+                            unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
+                        );
+                        Ok(Self {
+                            sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            digits,
+                        })
+                    }
+                }
+            }
+        }
+    )*)
+}
+
+plain_signed_checked_shr_impl!(i8 i16 i32 i64 i128 isize);
+
+macro_rules! plain_unsigned_checked_shr_impl {
+    ($($t:ty)*) => ($(
+        impl<Digit: RightShiftableDigit, const SEPARATOR: char, const SHIFT: usize> CheckedShr<$t>
+            for BigInt<Digit, SEPARATOR, SHIFT>
+        {
+            type Output = Result<Self, ShiftError>;
+
+            fn checked_shr(self, shift: $t) -> Self::Output {
+                debug_assert!(usize::BITS < <$t>::BITS || SHIFT < <$t>::MAX as usize);
+                if self.is_zero() {
+                    Ok(self)
+                } else {
+                    let (shift_quotient, shift_remainder) = shift.div_rem(SHIFT as $t);
+                    if (<$t>::BITS as usize) + 8 * size_of::<Digit>() >= (usize::BITS as usize)
+                        && shift_quotient >= ((usize::MAX / size_of::<Digit>()) as $t)
+                    {
+                        Err(ShiftError::TooLarge)
+                    } else {
+                        let digits = right_shift_digits::<Digit, SHIFT>(
+                            &self.digits,
+                            shift_quotient as usize,
+                            unsafe { Digit::try_from(shift_remainder as usize).unwrap_unchecked() },
+                        );
+                        Ok(Self {
+                            sign: self.sign * ((digits.len() > 1 || !digits[0].is_zero()) as Sign),
+                            digits,
+                        })
+                    }
+                }
+            }
+        }
+    )*)
+}
+
+plain_unsigned_checked_shr_impl!(u8 u16 u32 u64 u128 usize);
+
 impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> Sub
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -1127,20 +1495,88 @@ impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> SubAssign
     }
 }
 
+impl<Digit: DigitConvertibleFromFloat<f64>, const SEPARATOR: char, const SHIFT: usize> TryFrom<f64>
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Error = FromFloatConversionError;
+
+    fn try_from(mut value: f64) -> Result<Self, Self::Error> {
+        debug_assert!(usize::BITS < i32::BITS || SHIFT < (i32::MAX as usize));
+        if value.is_infinite() {
+            Err(FromFloatConversionError::Infinity)
+        } else if value.is_nan() {
+            Err(FromFloatConversionError::NaN)
+        } else if value.abs() < f64::one() {
+            Ok(Self::zero())
+        } else {
+            let mut sign = Sign::one();
+            if value.is_sign_negative() {
+                sign = -sign;
+                value = -value;
+            }
+            Ok(Self {
+                sign,
+                digits: digits_from_finite_positive_improper_float::<Digit, f64, SHIFT>(value),
+            })
+        }
+    }
+}
+
+impl<Digit: DigitConvertibleFromFloat<f64>, const SEPARATOR: char, const SHIFT: usize> TryFrom<f32>
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Error = FromFloatConversionError;
+
+    fn try_from(value: f32) -> Result<Self, Self::Error> {
+        Self::try_from(value as f64)
+    }
+}
+
 impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize> TryFrom<&str>
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
-    type Error = BigIntParsingError;
+    type Error = ParsingError;
 
     fn try_from(string: &str) -> Result<Self, Self::Error> {
         Self::new(string, 0)
     }
 }
 
-impl<Digit: Unitary + Zeroable, const SEPARATOR: char, const SHIFT: usize> Unitary
+impl<Digit: BinaryDigitConvertibleToFloat<f32>, const SEPARATOR: char, const SHIFT: usize>
+    TryFrom<BigInt<Digit, SEPARATOR, SHIFT>> for f32
+{
+    type Error = ToFloatConversionError;
+
+    fn try_from(value: BigInt<Digit, SEPARATOR, SHIFT>) -> Result<Self, Self::Error> {
+        match fraction_exponent_digits::<Digit, f32, SHIFT>(&value.digits) {
+            Some((fraction_modulus, exponent)) => {
+                Ok(((value.sign as f32) * fraction_modulus).ldexp(exponent))
+            }
+            None => Err(ToFloatConversionError::TooLarge),
+        }
+    }
+}
+
+impl<Digit: BinaryDigitConvertibleToFloat<f64>, const SEPARATOR: char, const SHIFT: usize>
+    TryFrom<BigInt<Digit, SEPARATOR, SHIFT>> for f64
+{
+    type Error = ToFloatConversionError;
+
+    fn try_from(value: BigInt<Digit, SEPARATOR, SHIFT>) -> Result<Self, Self::Error> {
+        match fraction_exponent_digits::<Digit, f64, SHIFT>(&value.digits) {
+            Some((fraction_modulus, exponent)) => {
+                Ok(((value.sign as f64) * fraction_modulus).ldexp(exponent))
+            }
+            None => Err(ToFloatConversionError::TooLarge),
+        }
+    }
+}
+
+impl<Digit: UnitaryDigit, const SEPARATOR: char, const SHIFT: usize> Unitary
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn one() -> Self {
+        debug_assert!(is_valid_shift::<Digit, SHIFT>());
         Self {
             sign: Sign::one(),
             digits: vec![Digit::one()],
@@ -1152,10 +1588,11 @@ impl<Digit: Unitary + Zeroable, const SEPARATOR: char, const SHIFT: usize> Unita
     }
 }
 
-impl<Digit: Zeroable, const SEPARATOR: char, const SHIFT: usize> Zeroable
+impl<Digit: ZeroableDigit, const SEPARATOR: char, const SHIFT: usize> Zeroable
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn zero() -> Self {
+        debug_assert!(is_valid_shift::<Digit, SHIFT>());
         Self {
             sign: Sign::zero(),
             digits: vec![Digit::zero()],
@@ -1166,3 +1603,7 @@ impl<Digit: Zeroable, const SEPARATOR: char, const SHIFT: usize> Zeroable
         self.sign.is_zero()
     }
 }
+
+const fn is_valid_shift<Digit: Oppose, const SHIFT: usize>() -> bool {
+    SHIFT < 8 * size_of::<Digit>() - (utils::are_same::<Digit, OppositionOf<Digit>>() as usize)
+}
