@@ -1,13 +1,15 @@
 use std::cmp::Ordering;
-use std::convert::TryFrom;
-use std::f64;
+use std::convert::{FloatToInt, TryFrom};
+use std::fmt::{Debug, Display, Formatter};
 
 use crate::traits::{
     AssigningAdditiveMonoid, AssigningBitwiseConjunctiveMagma, AssigningBitwiseDisjunctiveMonoid,
-    AssigningDivisivePartialMagma, AssigningMultiplicativeMonoid, AssigningShiftingLeftMonoid,
-    AssigningShiftingRightMonoid, AssigningSubtractiveMagma, DivisivePartialMagma, DoublePrecision,
-    DoublePrecisionOf, ModularPartialMagma, ModularSubtractiveMagma, Oppose, OppositionOf,
-    ShiftingLeftMonoid, SubtractiveMagma, Unitary, Zeroable,
+    AssigningBitwiseExclusiveDisjunctiveMonoid, AssigningDivisivePartialMagma,
+    AssigningMultiplicativeMonoid, AssigningShiftingLeftMonoid, AssigningShiftingRightMonoid,
+    AssigningSubtractiveMagma, BitwiseNegatableUnaryAlgebra, CheckedShl, DivisivePartialMagma,
+    DoublePrecision, DoublePrecisionOf, Float, ModularPartialMagma, ModularSubtractiveMagma,
+    Oppose, OppositionOf, ShiftingLeftMonoid, ShiftingRightMonoid, SubtractiveMagma, Unitary,
+    Zeroable,
 };
 use crate::utils;
 
@@ -19,7 +21,7 @@ pub trait AdditiveDigit = AssigningAdditiveMonoid
     + Copy
     + PartialOrd
     + ModularSubtractiveMagma
-    + Unitary;
+    + UnitaryDigit;
 
 pub trait BinaryDigit = AssigningAdditiveMonoid
     + AssigningBitwiseConjunctiveMagma
@@ -37,6 +39,18 @@ pub trait BinaryDigitConvertibleTo<Target> =
 pub trait BinaryDigitConvertibleToBinary<Target> =
     BinaryDigitDowncastableTo<Target> + BinaryDigitUpcastableTo<Target> where Target: TryFrom<Self>;
 
+pub trait BinaryDigitConvertibleToFloat<Target> = BinaryDigit
+    + Oppose
+    + DoublePrecision
+    + From<u8>
+    + TryFrom<DoublePrecisionOf<Self>>
+    + TryFrom<OppositionOf<Self>>
+where
+    DoublePrecisionOf<Self>: BinaryDigit,
+    OppositionOf<Self>: AssigningAdditiveMonoid + From<i8> + TryFrom<Self>,
+    Target: From<Self>,
+    usize: TryFrom<Self>;
+
 pub trait BinaryDigitConvertibleToNonBinary<Target> = Copy + DoublePrecision
 where
     Target: Copy + DoublePrecision + TryFrom<DoublePrecisionOf<Target>> + Zeroable,
@@ -58,32 +72,24 @@ where
     Target: DoublePrecision + TryFrom<DoublePrecisionOf<Target>>,
     DoublePrecisionOf<Target>: BinaryDigit + From<Self>;
 
+pub trait DigitConvertibleFromFloat<Source> =
+    Copy + ZeroableDigit where Source: FloatToInt<Self> + From<Self>;
+
 pub trait DisplayDigit = AssigningDivisivePartialMagma
-    + BinaryDigit
-    + From<u8>
+    + BinaryDigitConvertibleTo<Self>
     + ModularPartialMagma
-    + TryFrom<usize>
-    + BinaryDigitConvertibleTo<Self>;
+    + OppositiveDigit
+    + TryFrom<usize>;
 
 pub trait DivisibleDigit = BinaryDigit
     + DoublePrecision
     + From<u8>
-    + Oppose
     + TryFrom<DoublePrecisionOf<Self>>
     + TryFrom<OppositionOf<DoublePrecisionOf<Self>>>
     + TryFrom<usize>
+    + ZeroableDigit
 where
-    DoublePrecisionOf<Self>: AssigningAdditiveMonoid
-        + AssigningBitwiseConjunctiveMagma
-        + AssigningBitwiseDisjunctiveMonoid
-        + AssigningMultiplicativeMonoid
-        + AssigningShiftingLeftMonoid<usize>
-        + AssigningShiftingRightMonoid<usize>
-        + AssigningSubtractiveMagma
-        + Copy
-        + PartialOrd
-        + DivisivePartialMagma
-        + Oppose,
+    DoublePrecisionOf<Self>: BinaryDigit + DivisivePartialMagma + Oppose,
     OppositionOf<Self>:
         BinaryDigit + TryFrom<OppositionOf<DoublePrecisionOf<Self>>> + TryFrom<Self>,
     OppositionOf<DoublePrecisionOf<Self>>: BinaryDigit + From<Self> + From<OppositionOf<Self>>,
@@ -93,33 +99,17 @@ pub trait EuclidDivisibleDigit = AdditiveDigit + DivisibleDigit;
 
 pub trait ExponentiativeDigit = MultiplicativeDigit + BinaryDigitDowncastableTo<WindowDigit>;
 
-pub trait FromStrDigit = Copy
-    + DoublePrecision
-    + TryFrom<DoublePrecisionOf<Self>>
-    + TryFrom<u8>
-    + TryFrom<DoublePrecisionOf<u8>>
-    + TryFrom<DoublePrecisionOf<Self>>
-    + Zeroable
-where DoublePrecisionOf<Self>: BinaryDigit + From<u8> + TryFrom<usize>;
+pub trait FromStrDigit =
+    Oppose where u8: BinaryDigitConvertibleToBinary<Self> + NonBinaryDigitConvertibleToBinary<Self>;
 
-pub trait GcdDigit = BinaryDigit
-    + DoublePrecision
-    + From<u8>
-    + ModularSubtractiveMagma
-    + Oppose
-    + TryFrom<DoublePrecisionOf<Self>>
-    + TryFrom<OppositionOf<DoublePrecisionOf<Self>>>
-    + TryFrom<usize>
+pub trait GcdDigit = DivisibleDigit + ModularSubtractiveMagma
 where
-    DoublePrecisionOf<Self>: BinaryDigit + DivisivePartialMagma + ModularPartialMagma + Oppose,
-    OppositionOf<Self>:
-        BinaryDigit + TryFrom<OppositionOf<DoublePrecisionOf<Self>>> + TryFrom<Self>,
-    OppositionOf<DoublePrecisionOf<Self>>: BinaryDigit
-        + DivisivePartialMagma
-        + From<Self>
-        + From<OppositionOf<Self>>
-        + ModularPartialMagma,
-    usize: TryFrom<Self>;
+    DoublePrecisionOf<Self>: ModularPartialMagma + TryFrom<OppositionOf<DoublePrecisionOf<Self>>>,
+    OppositionOf<DoublePrecisionOf<Self>>:
+        DivisivePartialMagma + ModularPartialMagma + TryFrom<DoublePrecisionOf<Self>>;
+
+pub trait LeftShiftableDigit =
+    Debug + DivisibleDigit where DoublePrecisionOf<Self>: AssigningShiftingLeftMonoid<Self>;
 
 pub trait MultiplicativeDigit =
     AdditiveDigit + DoublePrecision + TryFrom<DoublePrecisionOf<Self>> + TryFrom<usize>
@@ -137,6 +127,20 @@ where
     Target: Copy + DoublePrecision + TryFrom<DoublePrecisionOf<Target>> + Zeroable,
     DoublePrecisionOf<Target>: BinaryDigit + From<Self> + From<Target> + TryFrom<usize>;
 
+pub trait OppositiveDigit = ZeroableDigit;
+
+pub trait RightShiftableDigit = AdditiveDigit
+    + AssigningBitwiseExclusiveDisjunctiveMonoid
+    + Debug
+    + DivisibleDigit
+    + ShiftingRightMonoid
+    + TryFrom<usize>
+where DoublePrecisionOf<Self>: AssigningShiftingLeftMonoid<Self>;
+
+pub trait UnitaryDigit = Unitary + OppositiveDigit;
+
+pub trait ZeroableDigit = Oppose + Zeroable;
+
 pub(crate) type Sign = i8;
 pub(crate) type WindowDigit = u8;
 
@@ -327,6 +331,160 @@ pub(crate) fn binary_digits_to_lesser_binary_base<
     result
 }
 
+pub enum CheckedDivApproximationError {
+    TooLarge,
+    ZeroDivision,
+}
+
+impl CheckedDivApproximationError {
+    fn description(&self) -> &str {
+        match self {
+            CheckedDivApproximationError::TooLarge => {
+                "Division result too large to be expressed as floating point."
+            }
+            CheckedDivApproximationError::ZeroDivision => "Division by zero is undefined.",
+        }
+    }
+}
+
+impl Debug for CheckedDivApproximationError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(self.description())
+    }
+}
+
+impl Display for CheckedDivApproximationError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
+pub(crate) fn checked_div_approximation<
+    Digit: BinaryDigitConvertibleToFloat<Output> + BitwiseNegatableUnaryAlgebra + DivisibleDigit,
+    Output: Float,
+    const SHIFT: usize,
+>(
+    dividend_digits: &[Digit],
+    divisor_digits: &[Digit],
+) -> Result<Output, CheckedDivApproximationError> {
+    if divisor_digits.len() == 1 && divisor_digits[0].is_zero() {
+        return Err(CheckedDivApproximationError::ZeroDivision);
+    }
+    if dividend_digits.len() == 1 && dividend_digits[0].is_zero() {
+        return Ok(Output::zero());
+    }
+    let dividend_digits_count = dividend_digits.len();
+    let divisor_digits_count = divisor_digits.len();
+    let dividend_is_small = dividend_digits_count <= (Output::MANTISSA_DIGITS / SHIFT)
+        || (dividend_digits_count == (Output::MANTISSA_DIGITS / SHIFT) + 1
+            && (dividend_digits[(Output::MANTISSA_DIGITS / SHIFT)]
+                >> (Output::MANTISSA_DIGITS % SHIFT))
+                .is_zero());
+    let divisor_is_small = divisor_digits_count <= (Output::MANTISSA_DIGITS / SHIFT)
+        || (divisor_digits_count == (Output::MANTISSA_DIGITS / SHIFT) + 1
+            && (divisor_digits[(Output::MANTISSA_DIGITS / SHIFT)]
+                >> (Output::MANTISSA_DIGITS % SHIFT))
+                .is_zero());
+    if dividend_is_small && divisor_is_small {
+        let reduced_dividend = reduce_digits_to_float::<Digit, Output, SHIFT>(dividend_digits);
+        let reduced_divisor = reduce_digits_to_float::<Digit, Output, SHIFT>(divisor_digits);
+        return Ok(reduced_dividend / reduced_divisor);
+    }
+    let digits_count_difference =
+        (dividend_digits_count as isize) - (divisor_digits_count as isize);
+    if digits_count_difference > (((usize::MAX / SHIFT) - 1) as isize) {
+        return Err(CheckedDivApproximationError::TooLarge);
+    } else if digits_count_difference < 1isize - ((usize::MAX / SHIFT) as isize) {
+        return Ok(Output::zero());
+    }
+    let bit_lengths_difference = digits_count_difference * (SHIFT as isize)
+        + (((utils::bit_length(dividend_digits[dividend_digits.len() - 1])) as isize)
+            - (utils::bit_length(divisor_digits[divisor_digits.len() - 1]) as isize));
+    if bit_lengths_difference > (Output::MAX_EXP as isize) {
+        return Err(CheckedDivApproximationError::TooLarge);
+    } else if bit_lengths_difference
+        < (Output::MIN_EXP as isize) - ((Output::MANTISSA_DIGITS as isize) - 1)
+    {
+        return Ok(Output::zero());
+    }
+    let shift = bit_lengths_difference.max(Output::MIN_EXP as isize)
+        - (Output::MANTISSA_DIGITS as isize)
+        - 2;
+    let mut inexact = false;
+    let mut quotient_digits = if shift <= 0 {
+        let shift_digits = ((-shift) as usize) / SHIFT;
+        if dividend_digits_count >= ((isize::MAX - 1) as usize) - shift_digits {
+            return Err(CheckedDivApproximationError::TooLarge);
+        }
+        let quotient_digits_count = dividend_digits_count + shift_digits + 1;
+        let mut quotient_data = vec![Digit::zero(); quotient_digits_count];
+        let remainder = shift_digits_left::<Digit, SHIFT>(
+            dividend_digits,
+            ((-shift) as usize) % SHIFT,
+            &mut quotient_data[shift_digits..],
+        );
+        quotient_data[dividend_digits_count + shift_digits] = remainder;
+        quotient_data
+    } else {
+        let mut shift_digits = (shift as usize) / SHIFT;
+        let quotient_digits_count = dividend_digits_count - shift_digits;
+        let mut quotient_data = vec![Digit::zero(); quotient_digits_count];
+        let remainder = shift_digits_right::<Digit, SHIFT>(
+            &dividend_digits[shift_digits..],
+            (shift as usize) % SHIFT,
+            &mut quotient_data,
+        );
+        if !remainder.is_zero() {
+            inexact = true;
+        }
+        while !inexact && shift_digits > 0 {
+            shift_digits -= 1;
+            if !dividend_digits[shift_digits].is_zero() {
+                inexact = true;
+            }
+        }
+        quotient_data
+    };
+    trim_leading_zeros(&mut quotient_digits);
+    if divisor_digits_count == 1 {
+        let (next_quotient_digits, remainder) =
+            div_rem_digits_by_digit::<Digit, SHIFT>(&quotient_digits, divisor_digits[0]);
+        quotient_digits = next_quotient_digits;
+        if !remainder.is_zero() {
+            inexact = true;
+        }
+    } else {
+        let (next_quotient_digits, remainder) =
+            div_rem_two_or_more_digits::<Digit, SHIFT>(&quotient_digits, divisor_digits);
+        quotient_digits = next_quotient_digits;
+        if remainder.len() > 1 || !remainder[0].is_zero() {
+            inexact = true;
+        }
+    }
+    let quotient_bit_length = ((quotient_digits.len() - 1) * SHIFT
+        + utils::bit_length(quotient_digits[quotient_digits.len() - 1]))
+        as isize;
+    let extra_bits = quotient_bit_length.max((Output::MIN_EXP as isize) - shift)
+        - (Output::MANTISSA_DIGITS as isize);
+    let mask = Digit::one() << ((extra_bits as usize) - 1);
+    let mut quotient_low_digit = quotient_digits[0] | Digit::from(inexact as u8);
+    if !(quotient_low_digit & mask).is_zero()
+        && !(quotient_low_digit & (Digit::from(3u8) * mask - Digit::from(1u8))).is_zero()
+    {
+        quotient_low_digit += mask;
+    }
+    quotient_digits[0] = quotient_low_digit & !(Digit::from(2u8) * mask - Digit::from(1u8));
+    let reduced_quotient = reduce_digits_to_float::<Digit, Output, SHIFT>(&quotient_digits);
+    if shift + quotient_bit_length >= (Output::MAX_EXP as isize)
+        && (shift + quotient_bit_length > (Output::MAX_EXP as isize)
+            || reduced_quotient == Output::one().ldexp(quotient_bit_length as i32))
+    {
+        Err(CheckedDivApproximationError::TooLarge)
+    } else {
+        Ok(reduced_quotient.ldexp(shift as i32))
+    }
+}
+
 pub(crate) fn checked_div<Digit: DivisibleDigit, const SHIFT: usize>(
     dividend: &[Digit],
     dividend_sign: Sign,
@@ -460,6 +618,28 @@ pub(crate) fn checked_rem_euclid<Digit: EuclidDivisibleDigit, const SHIFT: usize
     }
 }
 
+pub(crate) fn digits_from_finite_positive_improper_float<
+    Digit: Copy + ZeroableDigit,
+    Value: Float,
+    const SHIFT: usize,
+>(
+    value: Value,
+) -> Vec<Digit>
+where
+    Value: FloatToInt<Digit> + From<Digit>,
+{
+    let (fraction, exponent) = value.frexp();
+    let mut result = vec![Digit::zero(); ((exponent as usize) - 1) / SHIFT + 1];
+    let mut fraction = fraction.ldexp((exponent - 1) % (SHIFT as i32) + 1);
+    for index in (0..result.len()).rev() {
+        let digit = unsafe { Value::to_int_unchecked(fraction) };
+        result[index] = digit;
+        fraction -= Value::from(digit);
+        fraction = fraction.ldexp(SHIFT as i32);
+    }
+    result
+}
+
 #[inline]
 pub(crate) fn digits_lesser_than<Digit: PartialOrd>(left: &[Digit], right: &[Digit]) -> bool {
     left.len() < right.len()
@@ -588,118 +768,116 @@ pub(crate) fn div_rem_two_or_more_digits<Digit: DivisibleDigit, const SHIFT: usi
     (quotient, remainder)
 }
 
-fn non_binary_digits_to_greater_binary_base<
-    SourceDigit: NonBinaryDigitConvertibleToBinary<TargetDigit>,
-    TargetDigit,
-    const TARGET_SHIFT: usize,
+pub(crate) fn fraction_exponent_digits<
+    Digit: BinaryDigitConvertibleToFloat<Fraction>,
+    Fraction: Float,
+    const SHIFT: usize,
 >(
-    source: &[SourceDigit],
-    source_base: usize,
-) -> Vec<TargetDigit> {
-    let target_digit_mask = to_digit_mask::<DoublePrecisionOf<TargetDigit>>(TARGET_SHIFT);
-    static mut BASES_LOGS: [f64; 37] = [0.0; 37];
-    static mut INFIMUM_BASES_EXPONENTS: [usize; 37] = [0; 37];
-    static mut INFIMUM_BASES_POWERS: [usize; 37] = [0; 37];
-    if unsafe { BASES_LOGS[source_base] } == 0.0 {
-        let bases_log = (source_base as f64).ln() / ((1usize << TARGET_SHIFT) as f64).ln();
-        unsafe { BASES_LOGS[source_base] = bases_log };
-        let mut infimum_base_power = source_base;
-        let mut infimum_base_exponent: usize = 1;
-        loop {
-            let candidate: usize = infimum_base_power * source_base;
-            if candidate > 1usize << TARGET_SHIFT {
-                break;
-            }
-            infimum_base_power = candidate;
-            infimum_base_exponent += 1;
-        }
-        unsafe { INFIMUM_BASES_POWERS[source_base] = infimum_base_power };
-        unsafe { INFIMUM_BASES_EXPONENTS[source_base] = infimum_base_exponent };
+    digits: &[Digit],
+) -> Option<(Fraction, i32)> {
+    let mut result_digits =
+        vec![Digit::zero(); 2usize + (Fraction::MANTISSA_DIGITS + 1usize) / SHIFT];
+    let size = digits.len();
+    let mut bits_count = utils::bit_length(digits[digits.len() - 1]);
+    if size > (usize::MAX - 1) / SHIFT
+        && (size > (usize::MAX - 1) / SHIFT + 1 || bits_count > (usize::MAX - 1) % SHIFT + 1)
+    {
+        return None;
     }
-    let digits_count_upper_bound = (source.len() as f64) * unsafe { BASES_LOGS[source_base] } + 1.0;
-    let mut result = Vec::<TargetDigit>::with_capacity(digits_count_upper_bound as usize);
-    let infimum_base_exponent = unsafe { INFIMUM_BASES_EXPONENTS[source_base] };
-    let infimum_base_power = unsafe { INFIMUM_BASES_POWERS[source_base] };
-    let mut reversed_source = source.iter().rev();
-    while let Some(&digit) = reversed_source.next() {
-        let mut accumulator = DoublePrecisionOf::<TargetDigit>::from(digit);
-        let mut base_exponent: usize = 1;
-        while base_exponent < infimum_base_exponent {
-            if let Some(&digit) = reversed_source.next() {
-                base_exponent += 1;
-                unsafe {
-                    accumulator = accumulator
-                        * DoublePrecisionOf::<TargetDigit>::try_from(source_base)
-                            .unwrap_unchecked()
-                        + DoublePrecisionOf::<TargetDigit>::from(digit);
+    bits_count += (size - 1) * SHIFT;
+    let mut result_digits_count = if bits_count <= Fraction::MANTISSA_DIGITS + 2 {
+        let shift_digits = (Fraction::MANTISSA_DIGITS + 2 - bits_count) / SHIFT;
+        let shift_bits = (Fraction::MANTISSA_DIGITS + 2 - bits_count) % SHIFT;
+        let mut result_size = shift_digits;
+        let remainder = shift_digits_left::<Digit, SHIFT>(
+            digits,
+            shift_bits,
+            &mut result_digits[result_size..],
+        );
+        result_size += size;
+        result_digits[result_size] = remainder;
+        result_size += 1;
+        result_size
+    } else {
+        let mut shift_digits = (bits_count - Fraction::MANTISSA_DIGITS - 2) / SHIFT;
+        let shift_bits = (bits_count - Fraction::MANTISSA_DIGITS - 2) % SHIFT;
+        let remainder = shift_digits_right::<Digit, SHIFT>(
+            &digits[shift_digits..],
+            shift_bits,
+            &mut result_digits,
+        );
+        let result_size = size - shift_digits;
+        if remainder.is_zero() {
+            while shift_digits > 0 {
+                shift_digits -= 1;
+                if !digits[shift_digits].is_zero() {
+                    result_digits[0] |= Digit::one();
+                    break;
                 }
-            } else {
-                break;
             }
-        }
-        let base_power = if base_exponent == infimum_base_exponent {
-            infimum_base_power
         } else {
-            source_base.pow(base_exponent as u32)
-        };
-        for result_position in result.iter_mut() {
-            accumulator += DoublePrecisionOf::<TargetDigit>::from(*result_position)
-                * unsafe {
-                    DoublePrecisionOf::<TargetDigit>::try_from(base_power).unwrap_unchecked()
-                };
-            *result_position = unsafe {
-                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
-            };
-            accumulator >>= TARGET_SHIFT;
+            result_digits[0] |= Digit::one();
         }
-        if !accumulator.is_zero() {
-            result.push(unsafe { TargetDigit::try_from(accumulator).unwrap_unchecked() });
+        result_size
+    };
+    const HALF_EVEN_CORRECTION: [i8; 8] = [0, -1, -2, 1, 0, -1, 2, 1];
+    result_digits[0] = unsafe {
+        Digit::try_from(
+            OppositionOf::<Digit>::try_from(result_digits[0]).unwrap_unchecked()
+                + OppositionOf::<Digit>::from(
+                    HALF_EVEN_CORRECTION
+                        [usize::try_from(result_digits[0] & Digit::from(7u8)).unwrap_unchecked()],
+                ),
+        )
+        .unwrap_unchecked()
+    };
+    result_digits_count -= 1;
+    let mut fraction = Fraction::from(result_digits[result_digits_count]);
+    while result_digits_count > 0 {
+        result_digits_count -= 1;
+        fraction = fraction * Fraction::from((1usize << SHIFT) as f32)
+            + Fraction::from(result_digits[result_digits_count]);
+    }
+    fraction /= Fraction::from((1u64 << (Fraction::MANTISSA_DIGITS + 2)) as f32);
+    if fraction.is_one() {
+        if bits_count == usize::MAX {
+            return None;
         }
+        fraction = Fraction::from(0.5);
+        bits_count += 1;
     }
-    if result.is_empty() {
-        result.push(TargetDigit::zero());
+    let exponent = unsafe { i32::try_from(bits_count).unwrap_unchecked() };
+    if exponent > Fraction::MAX_EXP {
+        None
+    } else {
+        Some((fraction, exponent))
     }
-    result
 }
 
-fn non_binary_digits_to_lesser_binary_base<
-    SourceDigit: NonBinaryDigitConvertibleToBinary<TargetDigit>,
-    TargetDigit,
-    const TARGET_SHIFT: usize,
->(
-    source: &[SourceDigit],
-    source_base: usize,
-) -> Vec<TargetDigit> {
-    let target_digit_mask = to_digit_mask::<DoublePrecisionOf<TargetDigit>>(TARGET_SHIFT);
-    static mut BASES_LOGS: [f64; 37] = [0.0; 37];
-    if unsafe { BASES_LOGS[source_base] } == 0.0 {
-        let bases_log = (source_base as f64).ln() / ((1usize << TARGET_SHIFT) as f64).ln();
-        unsafe { BASES_LOGS[source_base] = bases_log };
-    }
-    let digits_count_upper_bound = (source.len() as f64) * unsafe { BASES_LOGS[source_base] } + 1.0;
-    let mut result = Vec::<TargetDigit>::with_capacity(digits_count_upper_bound as usize);
-    let source_base =
-        unsafe { DoublePrecisionOf::<TargetDigit>::try_from(source_base).unwrap_unchecked() };
-    for &digit in source.iter().rev() {
-        let mut accumulator = DoublePrecisionOf::<TargetDigit>::from(digit);
-        for result_position in result.iter_mut() {
-            accumulator += DoublePrecisionOf::<TargetDigit>::from(*result_position) * source_base;
-            *result_position = unsafe {
-                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
-            };
-            accumulator >>= TARGET_SHIFT;
-        }
-        while !accumulator.is_zero() {
-            result.push(unsafe {
-                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
-            });
-            accumulator >>= TARGET_SHIFT;
-        }
-    }
-    if result.is_empty() {
-        result.push(TargetDigit::zero());
-    }
+pub(crate) fn left_shift_digits<Digit: LeftShiftableDigit, const SHIFT: usize>(
+    digits: &[Digit],
+    shift_quotient: usize,
+    shift_remainder: Digit,
+) -> Option<Vec<Digit>> {
+    let mut result = Vec::<Digit>::new();
     result
+        .try_reserve_exact(shift_quotient + ((!shift_remainder.is_zero()) as usize) + digits.len())
+        .ok()?;
+    for _ in 0..shift_quotient {
+        result.push(Digit::zero());
+    }
+    let mut accumulator = DoublePrecisionOf::<Digit>::zero();
+    let digit_mask = to_digit_mask::<DoublePrecisionOf<Digit>>(SHIFT);
+    for &digit in digits {
+        accumulator |= DoublePrecisionOf::<Digit>::from(digit) << shift_remainder;
+        result.push(unsafe { Digit::try_from(accumulator & digit_mask).unwrap_unchecked() });
+        accumulator >>= SHIFT;
+    }
+    if !shift_remainder.is_zero() {
+        result.push(unsafe { Digit::try_from(accumulator).unwrap_unchecked() });
+    }
+    trim_leading_zeros(&mut result);
+    Some(result)
 }
 
 pub(crate) fn multiply_digits<Digit: MultiplicativeDigit, const SHIFT: usize>(
@@ -859,23 +1037,124 @@ pub(crate) fn negate_digits(digits: &mut [u8]) {
     }
 }
 
-pub(crate) fn trim_leading_zeros<Digit>(digits: &mut Vec<Digit>)
-where
-    Digit: Clone + Zeroable,
-{
-    let mut digits_count = digits.len();
-    while digits_count > 1 && digits[digits_count - 1].is_zero() {
-        digits_count -= 1;
+fn non_binary_digits_to_greater_binary_base<
+    SourceDigit: NonBinaryDigitConvertibleToBinary<TargetDigit>,
+    TargetDigit,
+    const TARGET_SHIFT: usize,
+>(
+    source: &[SourceDigit],
+    source_base: usize,
+) -> Vec<TargetDigit> {
+    let target_digit_mask = to_digit_mask::<DoublePrecisionOf<TargetDigit>>(TARGET_SHIFT);
+    static mut BASES_LOGS: [f64; 37] = [0.0; 37];
+    static mut INFIMUM_BASES_EXPONENTS: [usize; 37] = [0; 37];
+    static mut INFIMUM_BASES_POWERS: [usize; 37] = [0; 37];
+    if unsafe { BASES_LOGS[source_base] } == 0.0 {
+        let bases_log = (source_base as f64).ln() / ((1usize << TARGET_SHIFT) as f64).ln();
+        unsafe { BASES_LOGS[source_base] = bases_log };
+        let mut infimum_base_power = source_base;
+        let mut infimum_base_exponent: usize = 1;
+        loop {
+            let candidate: usize = infimum_base_power * source_base;
+            if candidate > 1usize << TARGET_SHIFT {
+                break;
+            }
+            infimum_base_power = candidate;
+            infimum_base_exponent += 1;
+        }
+        unsafe { INFIMUM_BASES_POWERS[source_base] = infimum_base_power };
+        unsafe { INFIMUM_BASES_EXPONENTS[source_base] = infimum_base_exponent };
     }
-    if digits_count != digits.len() {
-        digits.truncate(digits_count);
+    let digits_count_upper_bound = (source.len() as f64) * unsafe { BASES_LOGS[source_base] } + 1.0;
+    let mut result = Vec::<TargetDigit>::with_capacity(digits_count_upper_bound as usize);
+    let infimum_base_exponent = unsafe { INFIMUM_BASES_EXPONENTS[source_base] };
+    let infimum_base_power = unsafe { INFIMUM_BASES_POWERS[source_base] };
+    let mut reversed_source = source.iter().rev();
+    while let Some(&digit) = reversed_source.next() {
+        let mut accumulator = DoublePrecisionOf::<TargetDigit>::from(digit);
+        let mut base_exponent: usize = 1;
+        while base_exponent < infimum_base_exponent {
+            if let Some(&digit) = reversed_source.next() {
+                base_exponent += 1;
+                unsafe {
+                    accumulator = accumulator
+                        * DoublePrecisionOf::<TargetDigit>::try_from(source_base)
+                            .unwrap_unchecked()
+                        + DoublePrecisionOf::<TargetDigit>::from(digit);
+                }
+            } else {
+                break;
+            }
+        }
+        let base_power = if base_exponent == infimum_base_exponent {
+            infimum_base_power
+        } else {
+            source_base.pow(base_exponent as u32)
+        };
+        for result_position in result.iter_mut() {
+            accumulator += DoublePrecisionOf::<TargetDigit>::from(*result_position)
+                * unsafe {
+                    DoublePrecisionOf::<TargetDigit>::try_from(base_power).unwrap_unchecked()
+                };
+            *result_position = unsafe {
+                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
+            };
+            accumulator >>= TARGET_SHIFT;
+        }
+        if !accumulator.is_zero() {
+            result.push(unsafe { TargetDigit::try_from(accumulator).unwrap_unchecked() });
+        }
     }
+    if result.is_empty() {
+        result.push(TargetDigit::zero());
+    }
+    result
+}
+
+fn non_binary_digits_to_lesser_binary_base<
+    SourceDigit: NonBinaryDigitConvertibleToBinary<TargetDigit>,
+    TargetDigit,
+    const TARGET_SHIFT: usize,
+>(
+    source: &[SourceDigit],
+    source_base: usize,
+) -> Vec<TargetDigit> {
+    let target_digit_mask = to_digit_mask::<DoublePrecisionOf<TargetDigit>>(TARGET_SHIFT);
+    static mut BASES_LOGS: [f64; 37] = [0.0; 37];
+    if unsafe { BASES_LOGS[source_base] } == 0.0 {
+        let bases_log = (source_base as f64).ln() / ((1usize << TARGET_SHIFT) as f64).ln();
+        unsafe { BASES_LOGS[source_base] = bases_log };
+    }
+    let digits_count_upper_bound = (source.len() as f64) * unsafe { BASES_LOGS[source_base] } + 1.0;
+    let mut result = Vec::<TargetDigit>::with_capacity(digits_count_upper_bound as usize);
+    let source_base =
+        unsafe { DoublePrecisionOf::<TargetDigit>::try_from(source_base).unwrap_unchecked() };
+    for &digit in source.iter().rev() {
+        let mut accumulator = DoublePrecisionOf::<TargetDigit>::from(digit);
+        for result_position in result.iter_mut() {
+            accumulator += DoublePrecisionOf::<TargetDigit>::from(*result_position) * source_base;
+            *result_position = unsafe {
+                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
+            };
+            accumulator >>= TARGET_SHIFT;
+        }
+        while !accumulator.is_zero() {
+            result.push(unsafe {
+                TargetDigit::try_from(accumulator & target_digit_mask).unwrap_unchecked()
+            });
+            accumulator >>= TARGET_SHIFT;
+        }
+    }
+    if result.is_empty() {
+        result.push(TargetDigit::zero());
+    }
+    result
 }
 
 pub(crate) fn reduce_digits<Digit, Output, const SHIFT: usize>(digits: &[Digit]) -> Output
 where
     Digit: Copy,
-    Output: BinaryDigit + From<Digit>,
+    Output: BinaryDigit + Oppose + From<Digit>,
 {
     let mut result = Output::zero();
     for &digit in digits.iter().rev() {
@@ -884,6 +1163,61 @@ where
     result
 }
 
+pub(crate) fn checked_reduce_digits<Digit, Output, const SHIFT: usize>(
+    digits: &[Digit],
+) -> Option<Output>
+where
+    Digit: Copy,
+    Output: BinaryDigit
+        + Oppose
+        + Display
+        + CheckedShl<usize, Output = Option<Output>>
+        + TryFrom<Digit>,
+{
+    let mut result = Output::zero();
+    for &digit in digits.iter().rev() {
+        result = result.checked_shl(SHIFT)? | Output::try_from(digit).ok()?;
+    }
+    Some(result)
+}
+
+pub(crate) fn reduce_digits_to_float<Digit, Output, const SHIFT: usize>(digits: &[Digit]) -> Output
+where
+    Digit: Copy,
+    Output: Float + From<Digit>,
+{
+    let mut result = Output::zero();
+    for &digit in digits.iter().rev() {
+        result = result * Output::from((1u64 << SHIFT) as f32) + Output::from(digit);
+    }
+    result
+}
+
+pub(crate) fn right_shift_digits<Digit: RightShiftableDigit, const SHIFT: usize>(
+    digits: &[Digit],
+    shift_quotient: usize,
+    shift_remainder: Digit,
+) -> Vec<Digit> {
+    if digits.len() <= shift_quotient {
+        return vec![Digit::zero()];
+    }
+    let result_digits_count = digits.len() - shift_quotient;
+    let high_shift = SHIFT - unsafe { usize::try_from(shift_remainder).unwrap_unchecked() };
+    let low_mask = to_digit_mask::<Digit>(high_shift);
+    let high_mask = to_digit_mask::<Digit>(SHIFT) ^ low_mask;
+    let mut result = vec![Digit::zero(); result_digits_count];
+    let mut position = shift_quotient;
+    for index in 0..result_digits_count {
+        result[index] = (digits[position] >> shift_remainder) & low_mask;
+        if index + 1 < result_digits_count {
+            result[index] |= (digits[position + 1] << high_shift) & high_mask;
+        }
+        position += 1;
+    }
+    trim_leading_zeros(&mut result);
+    result
+}
+
 fn shift_digits_left<Digit, const SHIFT: usize>(
     input: &[Digit],
     shift: usize,
@@ -1126,3 +1460,16 @@ pub(crate) fn to_digit_mask<Digit: ShiftingLeftMonoid<usize> + SubtractiveMagma
 ) -> Digit {
     (Digit::one() << shift) - Digit::one()
 }
+
+pub(crate) fn trim_leading_zeros<Digit>(digits: &mut Vec<Digit>)
+where
+    Digit: Clone + Zeroable,
+{
+    let mut digits_count = digits.len();
+    while digits_count > 1 && digits[digits_count - 1].is_zero() {
+        digits_count -= 1;
+    }
+    if digits_count != digits.len() {
+        digits.truncate(digits_count);
+    }
+}
