@@ -1,13 +1,14 @@
 use std::cmp::Ordering;
-use std::fmt::{Display, Formatter};
+use std::convert::{FloatToInt, TryFrom, TryInto};
+use std::fmt::{Debug, Display, Formatter};
 use std::ops::{Add, AddAssign, Div, Mul, MulAssign, Neg, Sub, SubAssign};
 
 use crate::big_int::BigInt;
-use crate::digits::{AdditiveDigit, GcdDigit, MultiplicativeDigit};
+use crate::digits::{AdditiveDigit, GcdDigit, MultiplicativeDigit, UnitaryDigit};
 use crate::traits::{
-    Abs, AdditiveMonoid, CheckedDiv, CheckedPow, DivisivePartialMagma, GcdMagma,
-    ModularUnaryAlgebra, MultiplicativeMonoid, NegatableUnaryAlgebra, Oppositive, Pow,
-    SubtractiveMagma, Unitary, Zeroable,
+    Abs, AdditiveMonoid, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedPow, CheckedShl,
+    DivisivePartialMagma, Float, GcdMagma, Maybe, ModularUnaryAlgebra, MultiplicativeMonoid,
+    NegatableUnaryAlgebra, Oppositive, Pow, SubtractiveMagma, Unitary, Zeroable,
 };
 
 #[derive(Clone, Eq, PartialEq)]
@@ -16,6 +17,32 @@ pub struct Fraction<Component: Clone + Eq> {
     denominator: Component,
 }
 
+pub enum FromFloatConversionError {
+    Infinity,
+    NaN,
+}
+
+impl FromFloatConversionError {
+    fn description(&self) -> &str {
+        match self {
+            FromFloatConversionError::Infinity => "Conversion of infinity is undefined.",
+            FromFloatConversionError::NaN => "Conversion of NaN is undefined.",
+        }
+    }
+}
+
+impl Debug for FromFloatConversionError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(self.description())
+    }
+}
+
+impl Display for FromFloatConversionError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
 impl<Component: Clone + DivisivePartialMagma + Eq + GcdMagma + Oppositive> Fraction<Component> {
     pub fn new(mut numerator: Component, mut denominator: Component) -> Option<Self> {
         if denominator.is_zero() {
@@ -420,10 +447,6 @@ impl<Component: Clone + Eq + Unitary> PartialEq<Component> for Fraction<Componen
     fn eq(&self, other: &Component) -> bool {
         self.denominator.is_one() && self.numerator.eq(other)
     }
-
-    fn ne(&self, other: &Component) -> bool {
-        !self.denominator.is_one() || self.numerator.ne(other)
-    }
 }
 
 macro_rules! plain_partial_eq_fraction_impl {
@@ -432,26 +455,18 @@ macro_rules! plain_partial_eq_fraction_impl {
         fn eq(&self, other: &Fraction<Self>) -> bool {
             other.denominator.is_one() && other.numerator.eq(self)
         }
-
-        fn ne(&self, other: &Fraction<Self>) -> bool {
-            !other.denominator.is_one() || other.numerator.ne(self)
-        }
     }
     )*)
 }
 
 plain_partial_eq_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
-impl<Digit: Clone + Eq + Unitary + Zeroable, const SEPARATOR: char, const SHIFT: usize>
+impl<Digit: Clone + Eq + UnitaryDigit, const SEPARATOR: char, const SHIFT: usize>
     PartialEq<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn eq(&self, other: &Fraction<Self>) -> bool {
         other.denominator.is_one() && other.numerator.eq(self)
     }
-
-    fn ne(&self, other: &Fraction<Self>) -> bool {
-        !other.denominator.is_one() || other.numerator.ne(self)
-    }
 }
 
 impl<Component: Clone + Eq + MultiplicativeMonoid + PartialOrd> PartialOrd for Fraction<Component> {
@@ -720,6 +735,32 @@ impl<
     }
 }
 
+impl<Component: Clone + Eq + CheckedDivAsF32> TryInto<f32> for Fraction<Component> {
+    type Error = <<Component as CheckedDivAsF32>::Output as Maybe>::Error;
+
+    fn try_into(self) -> Result<f32, Self::Error> {
+        let maybe = self.numerator.checked_div_as_f32(self.denominator);
+        if maybe.is_result() {
+            Ok(maybe.result())
+        } else {
+            Err(maybe.error())
+        }
+    }
+}
+
+impl<Component: Clone + Eq + CheckedDivAsF64> TryInto<f64> for Fraction<Component> {
+    type Error = <<Component as CheckedDivAsF64>::Output as Maybe>::Error;
+
+    fn try_into(self) -> Result<f64, Self::Error> {
+        let maybe = self.numerator.checked_div_as_f64(self.denominator);
+        if maybe.is_result() {
+            Ok(maybe.result())
+        } else {
+            Err(maybe.error())
+        }
+    }
+}
+
 impl<Component: Clone + Eq + Unitary> Unitary for Fraction<Component> {
     fn one() -> Self {
         Self {
@@ -746,6 +787,76 @@ impl<Component: Clone + Eq + Unitary + Zeroable> Zeroable for Fraction<Component
     }
 }
 
+impl<Component: CheckedShl<u32> + Clone + Debug + Eq + Unitary + Zeroable> TryFrom<f32>
+    for Fraction<Component>
+where
+    f32: FloatToInt<Component>,
+{
+    type Error = FromFloatConversionError;
+
+    fn try_from(value: f32) -> Result<Self, Self::Error> {
+        if value.is_infinite() {
+            Err(FromFloatConversionError::Infinity)
+        } else if value.is_nan() {
+            Err(FromFloatConversionError::NaN)
+        } else {
+            let (numerator, denominator) =
+                finite_float_to_fraction_components::<Component, f32>(value);
+            Ok(Self {
+                numerator,
+                denominator,
+            })
+        }
+    }
+}
+
+impl<Component: CheckedShl<u32> + Clone + Debug + Eq + Unitary + Zeroable> TryFrom<f64>
+    for Fraction<Component>
+where
+    f64: FloatToInt<Component>,
+{
+    type Error = FromFloatConversionError;
+
+    fn try_from(value: f64) -> Result<Self, Self::Error> {
+        if value.is_infinite() {
+            Err(FromFloatConversionError::Infinity)
+        } else if value.is_nan() {
+            Err(FromFloatConversionError::NaN)
+        } else {
+            let (numerator, denominator) =
+                finite_float_to_fraction_components::<Component, f64>(value);
+            Ok(Self {
+                numerator,
+                denominator,
+            })
+        }
+    }
+}
+
+fn finite_float_to_fraction_components<
+    Component: Debug + Unitary + CheckedShl<u32>,
+    Value: Float + Display + FloatToInt<Component>,
+>(
+    value: Value,
+) -> (Component, Component) {
+    let (mut fraction, mut exponent) = value.frexp();
+    for _ in 0..300 {
+        if fraction == fraction.floor() {
+            break;
+        }
+        fraction *= Value::from(2.0f32);
+        exponent -= 1;
+    }
+    let mut numerator = unsafe { fraction.to_int_unchecked() };
+    let mut denominator = Component::one();
+    if exponent.is_negative() {
+        denominator = denominator.checked_shl((-exponent) as u32).result();
+    } else {
+        numerator = numerator.checked_shl(exponent as u32).result();
+    }
+    (numerator, denominator)
+}
+
 #[inline]
 fn normalize_components_moduli<Component: Clone + DivisivePartialMagma + GcdMagma>(
     numerator: Component,
