@@ -1,22 +1,27 @@
+#![feature(const_float_bits_conv)]
+#![feature(const_fn_trait_bound)]
+#![feature(convert_float_to_int)]
 #![feature(destructuring_assignment)]
 #![feature(option_result_unwrap_unchecked)]
 #![feature(specialization)]
 #![feature(trait_alias)]
+#![feature(try_reserve)]
 
 use std::cmp::Ordering;
-use std::convert::TryFrom;
+use std::convert::{TryFrom, TryInto};
 
 use pyo3::basic::CompareOp;
 use pyo3::class::PyObjectProtocol;
 use pyo3::exceptions::*;
 use pyo3::prelude::{pyclass, pymethods, pymodule, pyproto, PyModule, PyResult, Python};
-use pyo3::types::{PyBytes, PyLong, PyString};
-use pyo3::{ffi, AsPyPointer, Py, PyAny, PyErr, PyNativeType, ToPyObject};
+use pyo3::types::{PyBytes, PyFloat, PyLong, PyString};
+use pyo3::{ffi, AsPyPointer, Py, PyAny, PyErr, PyNativeType, PyRef, ToPyObject};
 use pyo3::{IntoPy, PyNumberProtocol, PyObject};
 
 use crate::traits::{
     Abs, CheckedDiv, CheckedDivEuclid, CheckedDivRemEuclid, CheckedPow, CheckedPowRemEuclid,
-    CheckedRemEuclid, FromStrRadix, Gcd, Oppositive, Pow, Unitary, Zeroable,
+    CheckedRemEuclid, CheckedShl, CheckedShr, FromStrRadix, Gcd, Oppositive, Pow, Unitary,
+    Zeroable,
 };
 
 pub mod big_int;
@@ -60,6 +65,19 @@ impl PyInt {
                     }
                 } else if value.is_instance::<PyInt>()? {
                     value.extract::<PyInt>()
+                } else if value.is_instance::<PyFloat>()? {
+                    Ok(PyInt(
+                        _BigInt::try_from(value.extract::<&PyFloat>()?.value()).map_err(
+                            |reason| match reason {
+                                big_int::FromFloatConversionError::Infinity => {
+                                    PyOverflowError::new_err(reason.to_string())
+                                }
+                                big_int::FromFloatConversionError::NaN => {
+                                    PyValueError::new_err(reason.to_string())
+                                }
+                            },
+                        )?,
+                    ))
                 } else {
                     let ptr = value.as_ptr();
                     let py = value.py();
@@ -101,26 +119,26 @@ impl PyInt {
         PyInt(self.0.clone().gcd(other.0))
     }
 
-    fn __ceil__(&self) -> PyObject {
-        big_int_to_py_long(&self.0)
+    fn __ceil__(slf: PyRef<Self>) -> PyRef<Self> {
+        slf
     }
 
-    fn __floor__(&self) -> PyObject {
-        big_int_to_py_long(&self.0)
+    fn __floor__(slf: PyRef<Self>) -> PyRef<Self> {
+        slf
     }
 
-    pub fn __getstate__(&self, py: Python) -> PyObject {
+    fn __getstate__(&self, py: Python) -> PyObject {
         PyBytes::new(py, &self.0.as_bytes()).to_object(py)
     }
 
-    pub fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
+    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
         state.extract::<&PyBytes>(py).map(|py_bytes| {
             self.0 = _BigInt::from_bytes(py_bytes.extract().unwrap());
         })
     }
 
-    fn __trunc__(&self) -> PyObject {
-        big_int_to_py_long(&self.0)
+    fn __trunc__(slf: PyRef<Self>) -> PyRef<Self> {
+        slf
     }
 }
 
@@ -138,19 +156,39 @@ fn big_int_to_py_long(value: &_BigInt) -> PyObject {
 #[pymethods]
 impl PyFraction {
     #[new]
-    fn new(_numerator: Option<PyInt>, _denominator: Option<PyInt>) -> PyResult<Self> {
-        match _Fraction::new(
-            _numerator
-                .map(|value| value.0)
-                .unwrap_or_else(_BigInt::zero),
-            _denominator
-                .map(|value| value.0)
-                .unwrap_or_else(_BigInt::one),
-        ) {
-            Some(value) => Ok(PyFraction(value)),
-            None => Err(PyZeroDivisionError::new_err(
-                UNDEFINED_DIVISION_ERROR_MESSAGE,
-            )),
+    fn new(_numerator: Option<&PyAny>, _denominator: Option<PyInt>) -> PyResult<Self> {
+        match _denominator {
+            Some(denominator) => match _Fraction::new(
+                _numerator
+                    .and_then(|value| value.extract::<PyInt>().ok())
+                    .map(|value| value.0)
+                    .unwrap_or_else(_BigInt::zero),
+                denominator.0,
+            ) {
+                Some(value) => Ok(PyFraction(value)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            },
+            None => Ok(PyFraction(match _numerator {
+                Some(value) => {
+                    if value.is_instance::<PyInt>()? {
+                        _Fraction::new(value.extract::<PyInt>()?.0, _BigInt::one()).unwrap()
+                    } else {
+                        _Fraction::try_from(value.extract::<f64>()?).map_err(
+                            |reason| match reason {
+                                fraction::FromFloatConversionError::Infinity => {
+                                    PyOverflowError::new_err(reason.to_string())
+                                }
+                                fraction::FromFloatConversionError::NaN => {
+                                    PyValueError::new_err(reason.to_string())
+                                }
+                            },
+                        )?
+                    }
+                }
+                None => _Fraction::zero(),
+            })),
         }
     }
 
@@ -164,7 +202,7 @@ impl PyFraction {
         PyInt(self.0.numerator().clone())
     }
 
-    pub fn __getstate__(&self, py: Python) -> PyObject {
+    fn __getstate__(&self, py: Python) -> PyObject {
         (
             self.numerator().__getstate__(py),
             self.denominator().__getstate__(py),
@@ -172,7 +210,7 @@ impl PyFraction {
             .to_object(py)
     }
 
-    pub fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
+    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
         state.extract::<(PyObject, PyObject)>(py).and_then(
             |(numerator_state, denominator_state)| {
                 let mut numerator = PyInt { 0: _BigInt::zero() };
@@ -205,6 +243,13 @@ impl PyNumberProtocol for PyInt {
         }
     }
 
+    fn __float__(&self) -> PyResult<PyObject> {
+        match f64::try_from(self.0.clone()) {
+            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
+            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
+        }
+    }
+
     fn __floordiv__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
         match lhs.0.checked_div_euclid(rhs.0) {
             Some(result) => Ok(PyInt(result)),
@@ -214,10 +259,24 @@ impl PyNumberProtocol for PyInt {
         }
     }
 
+    fn __int__(&self) -> PyObject {
+        big_int_to_py_long(&self.0)
+    }
+
     fn __invert__(&self) -> PyInt {
         PyInt(!self.0.clone())
     }
 
+    fn __lshift__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
+        lhs.0
+            .checked_shl(rhs.0)
+            .map(PyInt)
+            .map_err(|reason| match reason {
+                big_int::ShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
+                _ => PyValueError::new_err(reason.to_string()),
+            })
+    }
+
     fn __mod__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
         match lhs.0.checked_rem_euclid(rhs.0) {
             Some(result) => Ok(PyInt(result)),
@@ -267,6 +326,16 @@ impl PyNumberProtocol for PyInt {
         }
     }
 
+    fn __rshift__(lhs: PyInt, rhs: PyInt) -> PyResult<PyInt> {
+        lhs.0
+            .checked_shr(rhs.0)
+            .map(PyInt)
+            .map_err(|reason| match reason {
+                big_int::ShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
+                _ => PyValueError::new_err(reason.to_string()),
+            })
+    }
+
     fn __sub__(lhs: PyInt, rhs: PyInt) -> PyInt {
         PyInt(lhs.0 - rhs.0)
     }
@@ -291,10 +360,8 @@ fn hash(value: &_BigInt) -> usize {
         return if value.is_negative() {
             usize::MAX
                 - unsafe {
-                    usize::try_from(
-                        value.digits()[0] + <Digit as From<bool>>::from(value.digits()[0].is_one()),
-                    )
-                    .unwrap_unchecked()
+                    usize::try_from(value.digits()[0] + Digit::from(value.digits()[0].is_one()))
+                        .unwrap_unchecked()
                 }
                 + 1
         } else {
@@ -355,6 +422,13 @@ impl PyNumberProtocol for PyFraction {
         }
     }
 
+    fn __float__(&self) -> PyResult<PyObject> {
+        match <_Fraction as TryInto<f64>>::try_into(self.0.clone()) {
+            Ok(value) => Ok(Python::with_gil(|py| value.into_py(py))),
+            Err(reason) => Err(PyOverflowError::new_err(reason.to_string())),
+        }
+    }
+
     fn __mul__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
         let py = rhs.py();
         if rhs.is_instance::<PyFraction>()? {
