@@ -1,7 +1,9 @@
+use std::convert::TryFrom;
+use std::fmt::Debug;
 use std::num::ParseIntError;
 use std::ops::{
-    Add, AddAssign, BitAnd, BitAndAssign, BitOr, BitOrAssign, Div, DivAssign, Mul, MulAssign, Neg,
-    Rem, Shl, ShlAssign, Shr, ShrAssign, Sub, SubAssign,
+    Add, AddAssign, BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, Div, DivAssign,
+    Mul, MulAssign, Neg, Not, Rem, Shl, ShlAssign, Shr, ShrAssign, Sub, SubAssign,
 };
 
 pub trait AdditiveMonoid<Other = Self> = Add<Other, Output = Self> + Zeroable;
@@ -14,6 +16,9 @@ pub trait AssigningBitwiseConjunctiveMagma<Other = Self> =
 pub trait AssigningBitwiseDisjunctiveMonoid<Other = Self> =
     BitwiseDisjunctiveMonoid<Other> + BitOrAssign<Other>;
 
+pub trait AssigningBitwiseExclusiveDisjunctiveMonoid<Other = Self> =
+    BitwiseExclusiveDisjunctiveMonoid<Other> + BitXorAssign<Other>;
+
 pub trait AssigningDivisivePartialMagma<Divisor = Self> =
     DivisivePartialMagma<Divisor> + DivAssign<Divisor>;
 
@@ -32,8 +37,26 @@ pub trait BitwiseConjunctiveMagma<Other = Self> = BitAnd<Other, Output = Self> +
 
 pub trait BitwiseDisjunctiveMonoid<Other = Self> = BitOr<Other, Output = Self> + Zeroable;
 
+pub trait BitwiseExclusiveDisjunctiveMonoid<Other = Self> = BitXor<Other, Output = Self> + Zeroable;
+
+pub trait BitwiseNegatableUnaryAlgebra = Not<Output = Self>;
+
 pub trait DivisivePartialMagma<Divisor = Self> = Div<Divisor, Output = Self>;
 
+pub trait Float = AssigningAdditiveMonoid
+    + AssigningDivisivePartialMagma
+    + AssigningMultiplicativeMonoid
+    + AssigningSubtractiveMagma
+    + Copy
+    + Floor<Output = Self>
+    + FrExp<Output = (Self, i32)>
+    + From<f32>
+    + LdExp<i32, Output = Self>
+    + MantissaDigits
+    + MaxExp
+    + MinExp
+    + PartialEq;
+
 pub trait GcdMagma<Other = Self> = Gcd<Other, Output = Self>;
 
 pub trait ModularUnaryAlgebra = Abs<Output = Self>;
@@ -71,7 +94,7 @@ macro_rules! plain_abs_impl {
     )*)
 }
 
-plain_abs_impl!(i8 i16 i32 i64 i128 isize);
+plain_abs_impl!(f32 f64 i8 i16 i32 i64 i128 isize);
 
 pub trait DivEuclid<Divisor = Self> {
     type Output;
@@ -115,6 +138,56 @@ macro_rules! plain_checked_div_impl {
 
 plain_checked_div_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
+pub trait CheckedDivAsF32<Divisor = Self> {
+    type Output: Maybe<Result = f32>;
+
+    fn checked_div_as_f32(self, divisor: Divisor) -> Self::Output;
+}
+
+macro_rules! plain_checked_div_as_f32_impl {
+    ($($t:ty)*) => ($(
+        impl CheckedDivAsF32 for $t {
+            type Output = Option<f32>;
+
+            #[inline(always)]
+            fn checked_div_as_f32(self, divisor: Self) -> Self::Output {
+                if divisor.is_zero() {
+                    None
+                } else {
+                    Some((self as f32) / (divisor as f32))
+                }
+            }
+        }
+    )*)
+}
+
+plain_checked_div_as_f32_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+pub trait CheckedDivAsF64<Divisor = Self> {
+    type Output: Maybe<Result = f64>;
+
+    fn checked_div_as_f64(self, divisor: Divisor) -> Self::Output;
+}
+
+macro_rules! plain_checked_div_as_f64_impl {
+    ($($t:ty)*) => ($(
+        impl CheckedDivAsF64 for $t {
+            type Output = Option<f64>;
+
+            #[inline(always)]
+            fn checked_div_as_f64(self, divisor: Self) -> Self::Output {
+                if divisor.is_zero() {
+                    None
+                } else {
+                    Some((self as f64) / (divisor as f64))
+                }
+            }
+        }
+    )*)
+}
+
+plain_checked_div_as_f64_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 pub trait CheckedDivEuclid<Divisor = Self> {
     type Output;
 
@@ -379,31 +452,29 @@ macro_rules! plain_unsigned_checked_rem_euclid_inv_impl {
                     step_dividend = step_divisor;
                     step_divisor = remainder;
                     let subtrahend_modulus = quotient * candidate_modulus;
-                    (is_result_negative, result_modulus, candidate_modulus) = (
+                    (
+                        is_result_negative,
+                        result_modulus,
+                        (is_candidate_negative, candidate_modulus),
+                    ) = (
                         is_candidate_negative,
                         candidate_modulus,
                         if is_result_negative {
                             if is_candidate_negative {
                                 if result_modulus > subtrahend_modulus {
-                                    is_candidate_negative = true;
-                                    result_modulus - subtrahend_modulus
+                                    (true, result_modulus - subtrahend_modulus)
                                 } else {
-                                    is_candidate_negative = false;
-                                    subtrahend_modulus - result_modulus
+                                    (false, subtrahend_modulus - result_modulus)
                                 }
                             } else {
-                                is_candidate_negative = true;
-                                subtrahend_modulus + result_modulus
+                                (true, subtrahend_modulus + result_modulus)
                             }
                         } else if is_candidate_negative {
-                            is_candidate_negative = false;
-                            subtrahend_modulus + result_modulus
+                            (false, subtrahend_modulus + result_modulus)
                         } else if result_modulus > subtrahend_modulus {
-                            is_candidate_negative = false;
-                            result_modulus - subtrahend_modulus
+                            (false, result_modulus - subtrahend_modulus)
                         } else {
-                            is_candidate_negative = true;
-                            subtrahend_modulus - result_modulus
+                            (true, subtrahend_modulus - result_modulus)
                         },
                     );
                 }
@@ -444,6 +515,126 @@ macro_rules! plain_checked_rem_euclid_impl {
 
 plain_checked_rem_euclid_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
+pub trait CheckedShl<Shift = Self> {
+    type Output: Maybe<Result = Self>;
+
+    fn checked_shl(self, shift: Shift) -> Self::Output;
+}
+
+macro_rules! signed_checked_shl_impl {
+    ($t:ty, $f:ty) => {
+        impl CheckedShl<$f> for $t {
+            type Output = Option<$t>;
+
+            #[inline(always)]
+            fn checked_shl(self, other: $f) -> Self::Output {
+                if other < 0 {
+                    None
+                } else {
+                    if self.leading_zeros() < u32::try_from(other).ok()? {
+                        None
+                    } else {
+                        Some(self << other)
+                    }
+                }
+            }
+        }
+    };
+}
+
+macro_rules! unsigned_checked_shl_impl {
+    ($t:ty, $f:ty) => {
+        impl CheckedShl<$f> for $t {
+            type Output = Option<Self>;
+
+            #[inline(always)]
+            fn checked_shl(self, other: $f) -> Self::Output {
+                if self.leading_zeros() < u32::try_from(other).ok()? {
+                    None
+                } else {
+                    Some(self << other)
+                }
+            }
+        }
+    };
+}
+
+macro_rules! plain_checked_shl_impl {
+    ($($t:ty)*) => ($(
+        signed_checked_shl_impl! { $t, i8 }
+        signed_checked_shl_impl! { $t, i16 }
+        signed_checked_shl_impl! { $t, i32 }
+        signed_checked_shl_impl! { $t, i64 }
+        signed_checked_shl_impl! { $t, i128 }
+        signed_checked_shl_impl! { $t, isize }
+
+        unsigned_checked_shl_impl! { $t, u8 }
+        unsigned_checked_shl_impl! { $t, u16 }
+        unsigned_checked_shl_impl! { $t, u32 }
+        unsigned_checked_shl_impl! { $t, u64 }
+        unsigned_checked_shl_impl! { $t, u128 }
+        unsigned_checked_shl_impl! { $t, usize }
+    )*)
+}
+
+plain_checked_shl_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+pub trait CheckedShr<Shift = Self> {
+    type Output: Maybe<Result = Self>;
+
+    fn checked_shr(self, shift: Shift) -> Self::Output;
+}
+
+macro_rules! signed_checked_shr_impl {
+    ($t:ty, $f:ty) => {
+        impl CheckedShr<$f> for $t {
+            type Output = Option<$t>;
+
+            #[inline(always)]
+            fn checked_shr(self, other: $f) -> Self::Output {
+                if other < 0 {
+                    None
+                } else {
+                    <$t>::checked_shr(self, u32::try_from(other).ok()?)
+                }
+            }
+        }
+    };
+}
+
+macro_rules! unsigned_checked_shr_impl {
+    ($t:ty, $f:ty) => {
+        impl CheckedShr<$f> for $t {
+            type Output = Option<Self>;
+
+            #[inline(always)]
+            fn checked_shr(self, other: $f) -> Self::Output {
+                <$t>::checked_shr(self, u32::try_from(other).ok()?)
+            }
+        }
+    };
+}
+
+macro_rules! plain_checked_shr_impl {
+    ($($t:ty)*) => ($(
+        signed_checked_shr_impl! { $t, i8 }
+        signed_checked_shr_impl! { $t, i16 }
+        signed_checked_shr_impl! { $t, i32 }
+        signed_checked_shr_impl! { $t, i64 }
+        signed_checked_shr_impl! { $t, i128 }
+        signed_checked_shr_impl! { $t, isize }
+
+        unsigned_checked_shr_impl! { $t, u8 }
+        unsigned_checked_shr_impl! { $t, u16 }
+        unsigned_checked_shr_impl! { $t, u32 }
+        unsigned_checked_shr_impl! { $t, u64 }
+        unsigned_checked_shr_impl! { $t, u128 }
+        unsigned_checked_shr_impl! { $t, usize }
+    )*)
+}
+
+plain_checked_shr_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 pub trait DivRem<Divisor = Self> {
     type Output;
 
@@ -522,6 +713,83 @@ impl DoublePrecision for u64 {
     type Result = u128;
 }
 
+pub trait Floor {
+    type Output;
+
+    fn floor(self) -> Self::Output;
+}
+
+macro_rules! plain_floor_impl {
+    ($($t:ty)*) => ($(
+        impl Floor for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn floor(self) -> Self::Output {
+                <$t>::floor(self)
+            }
+        }
+    )*)
+}
+
+plain_floor_impl!(f32 f64);
+
+pub trait FrExp {
+    type Output;
+
+    fn frexp(self) -> Self::Output;
+}
+
+impl FrExp for f64 {
+    type Output = (Self, i32);
+
+    fn frexp(self) -> Self::Output {
+        let mut bits = self.to_bits();
+        let exponent_bits = ((bits >> 52) & 0x7ff) as i32;
+        if exponent_bits.is_zero() {
+            if !self.is_zero() {
+                const _0X1P64: f64 = f64::from_bits(0x43f0000000000000);
+                let (fraction, exponent) = (self * _0X1P64).frexp();
+                (fraction, exponent - 64)
+            } else {
+                (self, 0)
+            }
+        } else if exponent_bits == 0x7ff {
+            (self, 0)
+        } else {
+            let e = exponent_bits - 0x3fe;
+            bits &= 0x800fffffffffffff;
+            bits |= 0x3fe0000000000000;
+            (f64::from_bits(bits), e)
+        }
+    }
+}
+
+impl FrExp for f32 {
+    type Output = (Self, i32);
+
+    fn frexp(self) -> Self::Output {
+        let mut bits = self.to_bits();
+        let exponent_bits = ((bits >> 23) & 0xff) as i32;
+        if exponent_bits.is_zero() {
+            if !self.is_zero() {
+                const _0X1P64: f32 = f32::from_bits(0x5f800000);
+                let (fraction, exponent) = (self * _0X1P64).frexp();
+                (fraction, exponent - 64)
+            } else {
+                (self, 0)
+            }
+        } else if exponent_bits == 0xff {
+            (self, 0)
+        } else {
+            let exponent = exponent_bits - 0x7e;
+            bits &= 0x807fffff;
+            bits |= 0x3f000000;
+            (f32::from_bits(bits), exponent)
+        }
+    }
+}
+
 pub trait FromStrRadix: Sized {
     type Error;
 
@@ -565,70 +833,180 @@ macro_rules! plain_gcd_impl {
 
 plain_gcd_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
-pub trait Pow<Exponent> {
+pub trait LdExp<Exponent> {
     type Output;
 
-    fn pow(self, exponent: Exponent) -> Self::Output;
+    fn ldexp(self, exponent: Exponent) -> Self::Output;
 }
 
-macro_rules! plain_pow_impl {
-    ($($t:ty)*) => ($(
-        impl Pow<u32> for $t {
-            type Output = $t;
+impl LdExp<i32> for f32 {
+    type Output = Self;
 
-            #[inline(always)]
-            fn pow(self, exponent: u32) -> Self::Output {
-                <$t>::pow(self, exponent)
+    fn ldexp(mut self, mut exponent: i32) -> f32 {
+        const _0X1P127: f32 = f32::from_bits(0x7f000000);
+        const _0X1P_126: f32 = f32::from_bits(0x800000);
+        const _0X1P24: f32 = f32::from_bits(0x4b800000);
+        if exponent > 127 {
+            self *= _0X1P127;
+            exponent -= 127;
+            if exponent > 127 {
+                self *= _0X1P127;
+                exponent -= 127;
+                if exponent > 127 {
+                    exponent = 127;
+                }
             }
+        } else if exponent < -126 {
+            self *= _0X1P_126 * _0X1P24;
+            exponent += 126 - 24;
+            if exponent < -126 {
+                self *= _0X1P_126 * _0X1P24;
+                exponent += 126 - 24;
+                if exponent < -126 {
+                    exponent = -126;
+                }
+            }
+        }
+        self * f32::from_bits(((0x7f + exponent) as u32) << 23)
+    }
+}
+
+impl LdExp<i32> for f64 {
+    type Output = Self;
+
+    fn ldexp(mut self, mut exponent: i32) -> Self::Output {
+        if exponent > 1023 {
+            const _0X1P1023: f64 = f64::from_bits(0x7fe0000000000000);
+            self *= _0X1P1023;
+            exponent -= 1023;
+            if exponent > 1023 {
+                self *= _0X1P1023;
+                exponent -= 1023;
+                if exponent > 1023 {
+                    exponent = 1023;
+                }
+            }
+        } else if exponent < -1022 {
+            const _0X1P53: f64 = f64::from_bits(0x4340000000000000);
+            const _0X1P_1022: f64 = f64::from_bits(0x0010000000000000);
+            self *= _0X1P_1022 * _0X1P53;
+            exponent += 1022 - 53;
+            if exponent < -1022 {
+                self *= _0X1P_1022 * _0X1P53;
+                exponent += 1022 - 53;
+                if exponent < -1022 {
+                    exponent = -1022;
+                }
+            }
+        }
+        self * f64::from_bits(((0x3ff + exponent) as u64) << 52)
+    }
+}
+
+pub trait MantissaDigits {
+    const MANTISSA_DIGITS: usize;
+}
+
+macro_rules! plain_mantissa_digits_impl {
+    ($($t:ty)*) => ($(
+        impl MantissaDigits for $t {
+            const MANTISSA_DIGITS: usize = <$t>::MANTISSA_DIGITS as usize;
         }
     )*)
 }
 
-plain_pow_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+plain_mantissa_digits_impl!(f32 f64);
 
-pub trait RemEuclid<Divisor = Self> {
-    type Output;
-
-    fn rem_euclid(self, divisor: Divisor) -> Self::Output;
+pub trait MaxExp {
+    const MAX_EXP: i32;
 }
 
-macro_rules! plain_rem_euclid_impl {
+macro_rules! plain_max_exp_impl {
     ($($t:ty)*) => ($(
-        impl RemEuclid for $t {
-            type Output = $t;
-
-            #[inline(always)]
-            fn rem_euclid(self, divisor: Self) -> Self::Output {
-                <$t>::rem_euclid(self, divisor)
-            }
+        impl MaxExp for $t {
+            const MAX_EXP: i32 = <$t>::MAX_EXP;
         }
     )*)
 }
 
-plain_rem_euclid_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+plain_max_exp_impl!(f32 f64);
 
-pub trait Oppositive: NegatableUnaryAlgebra + Zeroable {
-    fn is_negative(&self) -> bool;
-    fn is_positive(&self) -> bool;
+pub trait Maybe {
+    type Result;
+    type Error;
+
+    fn error(self) -> Self::Error;
+    fn is_error(&self) -> bool;
+    fn is_result(&self) -> bool;
+    fn result(self) -> Self::Result;
 }
 
-macro_rules! plain_oppositive_impl {
-    ($($t:ty)*) => ($(
-        impl Oppositive for $t {
-            #[inline(always)]
-            fn is_negative(&self) -> bool {
-                <$t>::is_negative(*self)
-            }
+impl<T: Debug> Maybe for Option<T> {
+    type Result = T;
+    type Error = Option<T>;
 
-            #[inline(always)]
-            fn is_positive(&self) -> bool {
-                <$t>::is_positive(*self)
-            }
+    #[inline(always)]
+    fn error(self) -> Self::Error {
+        match self {
+            Some(value) => panic!("called `Option::error()` on `Some{:?}` value", value),
+            None => None,
+        }
+    }
+
+    #[inline(always)]
+    fn is_error(&self) -> bool {
+        Option::<T>::is_none(self)
+    }
+
+    #[inline(always)]
+    fn is_result(&self) -> bool {
+        Option::<T>::is_some(self)
+    }
+
+    #[inline(always)]
+    fn result(self) -> Self::Result {
+        Option::<T>::unwrap(self)
+    }
+}
+
+impl<T: Debug, E: Debug> Maybe for Result<T, E> {
+    type Result = T;
+    type Error = E;
+
+    #[inline(always)]
+    fn error(self) -> Self::Error {
+        Result::<T, E>::unwrap_err(self)
+    }
+
+    #[inline(always)]
+    fn is_error(&self) -> bool {
+        Result::<T, E>::is_err(self)
+    }
+
+    #[inline(always)]
+    fn is_result(&self) -> bool {
+        Result::<T, E>::is_ok(self)
+    }
+
+    #[inline(always)]
+    fn result(self) -> Self::Result {
+        Result::<T, E>::unwrap(self)
+    }
+}
+
+pub trait MinExp {
+    const MIN_EXP: i32;
+}
+
+macro_rules! plain_min_exp_impl {
+    ($($t:ty)*) => ($(
+        impl MinExp for $t {
+            const MIN_EXP: i32 = <$t>::MIN_EXP;
         }
     )*)
 }
 
-plain_oppositive_impl!(i8 i16 i32 i64 i128 isize);
+plain_min_exp_impl!(f32 f64);
 
 pub trait Oppose {
     type Result: Oppositive;
@@ -682,6 +1060,71 @@ impl Oppose for usize {
     type Result = isize;
 }
 
+pub trait Oppositive: NegatableUnaryAlgebra + Zeroable {
+    fn is_negative(&self) -> bool;
+    fn is_positive(&self) -> bool;
+}
+
+macro_rules! plain_oppositive_impl {
+    ($($t:ty)*) => ($(
+        impl Oppositive for $t {
+            #[inline(always)]
+            fn is_negative(&self) -> bool {
+                <$t>::is_negative(*self)
+            }
+
+            #[inline(always)]
+            fn is_positive(&self) -> bool {
+                <$t>::is_positive(*self)
+            }
+        }
+    )*)
+}
+
+plain_oppositive_impl!(i8 i16 i32 i64 i128 isize);
+
+pub trait Pow<Exponent> {
+    type Output;
+
+    fn pow(self, exponent: Exponent) -> Self::Output;
+}
+
+macro_rules! plain_pow_impl {
+    ($($t:ty)*) => ($(
+        impl Pow<u32> for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn pow(self, exponent: u32) -> Self::Output {
+                <$t>::pow(self, exponent)
+            }
+        }
+    )*)
+}
+
+plain_pow_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+pub trait RemEuclid<Divisor = Self> {
+    type Output;
+
+    fn rem_euclid(self, divisor: Divisor) -> Self::Output;
+}
+
+macro_rules! plain_rem_euclid_impl {
+    ($($t:ty)*) => ($(
+        impl RemEuclid for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn rem_euclid(self, divisor: Self) -> Self::Output {
+                <$t>::rem_euclid(self, divisor)
+            }
+        }
+    )*)
+}
+
+plain_rem_euclid_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 pub trait Unitary {
     fn one() -> Self;
 
@@ -692,7 +1135,7 @@ macro_rules! plain_unitary_impl {
     ($($t:ty)*) => ($(
         impl Unitary for $t {
             #[inline(always)]
-            fn one() -> $t {1}
+            fn one() -> $t {1 as $t}
 
             #[inline(always)]
             fn is_one(&self) -> bool {
@@ -702,7 +1145,7 @@ macro_rules! plain_unitary_impl {
     )*)
 }
 
-plain_unitary_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+plain_unitary_impl!(f32 f64 i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 pub trait WrappingSub<Subtrahend = Self> {
     type Output;
@@ -735,7 +1178,7 @@ macro_rules! plain_zero_impl {
     ($($t:ty)*) => ($(
         impl Zeroable for $t {
             #[inline(always)]
-            fn zero() -> $t {0}
+            fn zero() -> $t {0 as $t}
 
             #[inline(always)]
             fn is_zero(&self) -> bool {
@@ -745,7 +1188,7 @@ macro_rules! plain_zero_impl {
     )*)
 }
 
-plain_zero_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+plain_zero_impl!(f32 f64 i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 pub type DoublePrecisionOf<T> = <T as DoublePrecision>::Result;
 pub type OppositionOf<T> = <T as Oppose>::Result;
