@@ -11,7 +11,7 @@ def test_alternatives(first: Int, second: Int) -> None:
     assert first - second == first + (-second)
 
 
-@given(strategies.ints_with_builtin_ints, strategies.ints_with_builtin_ints)
+@given(strategies.ints_with_builtins, strategies.ints_with_builtins)
 def test_connection_with_builtin(first_with_builtin: IntWithBuiltin,
                                  second_with_builtin: IntWithBuiltin
                                  ) -> None:
